from yxspkg import tecfile

def q2szplt(szplt_name,g,q,P,T,lref=1,gamma=1.4,R=287.14):
    if isinstance(g,str):
        g = tecfile.read(g) 
    if isinstance(q,str):
        q = tecfile.read(q)
    Tref = T
    Pref = P
    Rhoref = Pref/(R*Tref)
    aref = (gamma*R*Tref)**0.5
    result = dict()
    cp = gamma/(gamma-1)*R
    for zone_id,(mesh,data) in enumerate(zip(g,q)):
        density = data[0]
        u = data[1]/density 
        v = data[2]/density 
        w = data[3]/density
        p = (data[4]-0.5*(u*u+v*v+w*w)*density)*(gamma-1)
        T = ((gamma*p)/density)*Tref
        u *= aref
        v *= aref
        w *= aref
        density =density * Rhoref
        velocity = (u*u+v*v+w*w)**0.5
        p *= Rhoref*aref*aref
        
        t_total = T + velocity**2/(2*cp)
        ptp = (t_total/T)**(gamma/(gamma-1))
        p_total = p*ptp
        new_dict ={}
        new_dict['CoordinateX'] = mesh['X']
        new_dict['CoordinateY'] = mesh['Y']
        new_dict['CoordinateZ'] = mesh['Z']
        new_dict['Temperature']=T 
        new_dict['TemperatureStagnation']=t_total
        new_dict['Pressure']=p 
        new_dict['PressureStagnation'] = p_total
        new_dict['VelocityX'] = u 
        new_dict['VelocityY'] = v 
        new_dict['VelocityZ'] = w 
        new_dict['VelocityMagnitude'] = velocity
        new_dict['Density'] = density
        
        
        
        result['zone{}'.format(zone_id+1)] = new_dict
    if szplt_name:
        tecfile.write(szplt_name,result)
    return result
def q2nq(q,density0,temperature,gamma=1.4,R=287.14):
    acu_vel = (gamma*R*temperature)**0.5
    result = []
    for data in q:
        density = data[0]
        u = data[1]/density 
        v = data[2]/density 
        w = data[3]/density
        p = (data[4]-0.5*(u*u+v*v+w*w)*density)*(gamma-1)
        u *= acu_vel 
        v *= acu_vel 
        w *= acu_vel
        density =density * density0 
        p *= density0*acu_vel*acu_vel
        new_dict ={}
        new_dict[0] = density
        new_dict[1] = u 
        new_dict[2] = v 
        new_dict[3] = w 
        new_dict[4] = p 
        result.append(new_dict)
    return result
def nq2q(nq,density0,temperature,velocity,gamma=1.4,R=287.14):
    acu_vel = (gamma*R*temperature)**0.5
    result = []
    for data in nq:
        density = data[0]/density0 
        u = data[1] / acu_vel
        v = data[2] / acu_vel
        w = data[3] / acu_vel
        p = data[4] /(density0*acu_vel**2)
        e = p/(gamma-1)+0.5*density*(u*u+v*v+w*w)
        u *= density 
        v *= density 
        w *= density
        new_dict = dict() 
        new_dict[0] = density 
        new_dict[1] = u 
        new_dict[2] = v 
        new_dict[3] = w 
        new_dict[4] = e 
        new_dict['time'] = 0.0
        new_dict['mach'] = 0.0
        new_dict['Re'] = 0.0 
        new_dict['alpha'] = 0.0
        result.append(new_dict)
    return result
def szplt2nq(szplt):
    t = szplt
    keys = list(t.keys())
    keys.sort(key = lambda x:int(x[6:]))
    nqfile = []
    gfile = []
    for i in keys:
        data = t[i]
        g = {'X':data['CoordinateX'],'Y':data['CoordinateY'],'Z':data['CoordinateZ']}
        gfile.append(g)
        q_dict={
            0:data['Density'],
            1:data['VelocityXYZ.vx'],
            2:data['VelocityXYZ.vy'],
            3:data['VelocityXYZ.vz'],
            4:data['Pressure'],
            'mach':0.0,
            'alpha':0.0,
            'Re':1.0,
            'time':1.0
        }
        nqfile.append(q_dict)
    return nqfile,gfile

if __name__=='__main__':
    pass


from pathlib import Path
import time
import math
import numpy as np 
from yxspkg import tecfile
from functools import wraps
import re
from scipy.interpolate import RegularGridInterpolator
def result_wrapper(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        t = func(*args, **kwargs)
        if t is None:
            return np.array([])
        t = [i.split() for i in t]
        try:
            t = np.array(t,dtype='float64')
        except:
            print(t)
        if not t.any():
            return t
        self = args[0]
        if self.blk is None:
            #取self.blk 为blk序列中的众数
            m = t[:,1]
            m = m[1:] - m[:-1]
            m = t[:,1][np.where(m==0)]
            self.blk = m[0]

        t = t[np.where(np.abs(t[:,1] - self.blk)<0.01)]
        return t
    return wrapper
class Residual:
    def __init__(self,filename):
        self.filename = Path(filename)
        self.pos=-1
        self.titles = None
        self.re_res = None
        self.latest_mtime = 0
        self.re_res = self.get_re_res()
        self.blk = None
    def get_re_res(self):
        fp = open(self.filename,'r')
        for i in fp:
            if i.find('total res.')!=-1:
                self.titles = re.split(' {2,}',i)
                n = len(self.titles) - 3
                return re.compile('\s+\d+\s+\d+\s+\d+\s+([-\.\d]+E[\+-]\d+\s+){{{n}}}'.format(n = n))
        return None

    @result_wrapper
    def get(self):
        mtime = self.filename.lstat().st_mtime
        if mtime <= self.latest_mtime:
            return []
        self.latest_mtime = mtime
        fp = open(self.filename,'r')
        if self.re_res is None:
            t = self.get_re_res()
            if t is not None:
                self.re_res = t
            else:
                return None
        fp.seek(self.pos+1,0)
        res_list = [i for i in fp if self.re_res.match(i)]
        self.pos = fp.tell()
        return res_list
    def get_history(self):
        fp = open(self.filename,'r')
        is_title = False
        title_re = re.compile(' *iteration *residual *CL *CD *CY *CMY.*')
        data_re  = re.compile('( +\d+)( +[\.\dEe\+\-]+)( +[\.\dEe\+\-]+)( +[\.\dEe\+\-]+)( +[\.\dEe\+\-]+)( +[\.\dEe\+\-]+)')
        result = []
        for line in fp:
            if title_re.match(line):
                is_title = True 
                continue
            if is_title==True:
                t = data_re.findall(line)
                if t:
                    result.append(t)
                else:
                    break
        if not is_title:
            return np.array([])
        result = np.array(result,dtype = 'float32').reshape((len(result),-1))
        if len(result) > 200:
            k = len(result) // 200
            result = result[::k,:]
        result = np.hstack((result[:,-2:],result,(result[:,-2:])))
        result[:,4] = result[:,3]
        return result
def q2szplt(szplt_name,g,q,P,T,lref=1,gamma=1.4,R=287.14):
    if isinstance(g,str):
        g = tecfile.read(g) 
    if isinstance(q,str):
        q = tecfile.read(q)
    Tref = T
    Pref = P
    Rhoref = Pref/(R*Tref)
    aref = (gamma*R*Tref)**0.5
    result = dict()
    cp = gamma/(gamma-1)*R
    for zone_id,(mesh,data) in enumerate(zip(g,q)):
        density = data[0]
        u = data[1]/density 
        v = data[2]/density 
        w = data[3]/density
        p = (data[4]-0.5*(u*u+v*v+w*w)*density)*(gamma-1)
        T = ((gamma*p)/density)*Tref
        u *= aref
        v *= aref
        w *= aref
        density =density * Rhoref
        velocity = (u*u+v*v+w*w)**0.5
        p *= Rhoref*aref*aref
        
        t_total = T + velocity**2/(2*cp)
        ptp = (t_total/T)**(gamma/(gamma-1))
        p_total = p*ptp
        new_dict ={}
        new_dict['X'] = mesh['X']
        new_dict['Y'] = mesh['Y']
        new_dict['Z'] = mesh['Z']
        new_dict['density'] = density
        new_dict['velocity_x'] = u 
        new_dict['velocity_y'] = v 
        new_dict['velocity_z'] = w 
        new_dict['velocity_mag'] = velocity
        new_dict['pressure']=p 
        new_dict['total_pressure'] = p_total
        new_dict['temperature']=T 
        new_dict['total_temperature']=t_total
        result['zone_{}'.format(zone_id)] = new_dict
    tecfile.write(szplt_name,result)
    return result
def q2nq(q,density0,temperature,gamma=1.4,R=287.14):
    acu_vel = (gamma*R*temperature)**0.5
    result = []
    for data in q:
        density = data[0]
        u = data[1]/density 
        v = data[2]/density 
        w = data[3]/density
        p = (data[4]-0.5*(u*u+v*v+w*w)*density)*(gamma-1)
        u *= acu_vel 
        v *= acu_vel 
        w *= acu_vel
        density =density * density0 
        p *= density0*acu_vel*acu_vel
        new_dict ={}
        new_dict[0] = density
        new_dict[1] = u 
        new_dict[2] = v 
        new_dict[3] = w 
        new_dict[4] = p 
        result.append(new_dict)
    return result
def nq2q(nq,density0,temperature,velocity,gamma=1.4,R=287.14):
    acu_vel = (gamma*R*temperature)**0.5
    result = []
    for data in nq:
        density = data[0]/density0 
        u = data[1] / acu_vel
        v = data[2] / acu_vel
        w = data[3] / acu_vel
        p = data[4] /(density0*acu_vel**2)
        e = p/(gamma-1)+0.5*density*(u*u+v*v+w*w)
        u *= density 
        v *= density 
        w *= density
        new_dict = dict() 
        new_dict[0] = density 
        new_dict[1] = u 
        new_dict[2] = v 
        new_dict[3] = w 
        new_dict[4] = e 
        new_dict['time'] = 0.0
        new_dict['mach'] = 0.0
        new_dict['Re'] = 0.0 
        new_dict['alpha'] = 0.0
        result.append(new_dict)
    return result
def szplt2nq(szplt):
    t = szplt
    keys = list(t.keys())
    keys.sort(key = lambda x:int(x[6:]))
    nqfile = []
    gfile = []
    for i in keys:
        data = t[i]
        g = {'X':data['CoordinateX'],'Y':data['CoordinateY'],'Z':data['CoordinateZ']}
        gfile.append(g)
        q_dict={
            0:data['Density'],
            1:data['VelocityXYZ.vx'],
            2:data['VelocityXYZ.vy'],
            3:data['VelocityXYZ.vz'],
            4:data['Pressure'],
            'mach':0.0,
            'alpha':0.0,
            'Re':1.0,
            'time':1.0
        }
        nqfile.append(q_dict)
    return nqfile,gfile
def interpolateblock(data,toshape,method='linear'):
    fromshape = data.shape
    if fromshape == toshape:
        return data    
    x = np.linspace(0,1,fromshape[0])
    y = np.linspace(0,1,fromshape[1])
    z = np.linspace(0,1,fromshape[2])
    inter_fun = RegularGridInterpolator((x,y,z),data,method=method)
    x2 = np.linspace(0,1,toshape[0])
    y2 = np.linspace(0,1,toshape[1])
    z2 = np.linspace(0,1,toshape[2])
    f = np.meshgrid(x2,y2,z2,indexing='ij')
    t = np.stack(f,axis=3)
    k = inter_fun(t)
    return k
def rotate_block(block,angle):
    sint = math.sin(angle)
    cost = math.cos(angle)
    x = block['X']*cost-sint*block['Y']
    block['Y'] = sint*block['X'] + cost*block['Y']
    block['X'] = x 
    return block

if __name__=='__main__':
    # f = '/home/yxs/F/learning/CFL3D/323_304/323_304_e2/rotate323_cfl3d.out'
    # f2 = '/home/yxs/F/pythonAPP/cfl3d_pyui/RS_work/rotate323_cfl3d.out'
    # res = Residual(f2)
    # k = res.get_history()
    # print(k.shape)
    # for i in k:
    #     print(i)
    # print(res.titles)
    # print(k)

    # p = Path('/home/yxs/F/learning/CFL3D/CFL3D/U-Duct-e6-SA_6.7.2/cfl3d.out')
    # print(p.lstat())
    from yxspkg import tecfile
    q2szplt(
        '/home/yxs/F/learning/CFL3D/323_0409/ok/steady_323_0409_ok_freeze(复件)/rotate323_0409_lref1_output.szplt',
        '/home/yxs/F/learning/CFL3D/323_0409/ok/steady_323_0409_ok_freeze(复件)/rotate323_0409_lref1_output.g',
        '/home/yxs/F/learning/CFL3D/323_0409/ok/steady_323_0409_ok_freeze(复件)/q_and_boundary.q',
        90748.287,279.215
    )
   
    # def double_shape(shape):
    #     i,j,k = shape 
    #     return i,321,101
    # a = tecfile.read('/home/yxs/F/pythonAPP/julia_CFD/transdiff_p3d.g')
    # a = [{key:interpolateblock(i[key],double_shape(i[key].shape)) for key in "XYZ"} for i in a]
    # tecfile.write("/home/yxs/F/pythonAPP/julia_CFD/transdiff_p3d2.g",a)
    # qfile = '/home/yxs/F/learning/CFL3D/323_0409/numeca_0409_323_steady/new_grid_323_0409_nogap/323_0409_steady/323_0409_steady/323_0409_steady_323_0411_STEADY/323_0409_steady_323_0411_STEADY.q'
    # q = tecfile.read(qfile)
    # gfile = '/home/yxs/F/learning/CFL3D/323_0409/test2/rotate323_0409_mesh.g'
    # g = tecfile.read(gfile)
    # bs = [rotate_block(g[i],-4/180*3.1415) for i in range(15,15+12)]
    # g = g[:15]+bs+g[27:]
    # print(len(g))
    # tecfile.write('/home/yxs/F/learning/CFL3D/323_0409/test2/rotate323_0409_mesh2.g',g)
    # print(len(g),len(q))
    # for i,j in zip(g,q):
    #     toshape = i['X'].shape
    #     for key,value in j.items():
    #         if isinstance(key,str):
    #             continue
    #         todata = interpolateblock(value,toshape)
    #         # print(todata.dtype,key)
    #         j[key] = todata
    # print(q[0][0].shape,g[0]['X'].shape)
    # q = tecfile.read('/home/yxs/F/learning/CFL3D/323_0409/323_new_bc2009_unsteady/1/mesh_0006.q')
    # t = q2nq(q,1.34115,232.558)
    # tecfile.write('/home/yxs/F/learning/CFL3D/323_0409/323_new_bc2009_unsteady/1/mesh_0006_nq.q',t)
    # tecfile.write('/home/yxs/F/learning/CFL3D/323_0409/cfl3d_test_0409_323grid/rotate323_0409_mesh.q',q)
    # nq = tecfile.read('/home/yxs/F/pythonAPP/cfl3d_pyui/initial_field_test/init_nq.q')
    # q2 = nq2q(nq,1.2,300,200)
    # for i,j in zip(q,q2):
    #     for m in range(5):
    #         d1 = i[m]
    #         d2 = j[m]
    #         d = np.abs(d1-d2)
    #         d = d.max() - d.min()
    #         if d>1e-6:
    #             raise Exception('error')
    # tecfile.write('init_test.q',q2)
    # szplt = tecfile.read('/home/yxs/F/learning/CFL3D/323_0409/numeca_0409_323_steady/new_grid_323_0409_nogap/323_0409_steady/323_0409_steady/323_0409_steady_323_0411_STEADY/323_0409_steady_323_0411_STEADY.szplt')
    # nq,g = szplt2nq(szplt)
    # q = nq2q(nq,1.2,280,122)
    # tecfile.write('/home/yxs/F/learning/CFL3D/323_0409/numeca_0409_323_steady/new_grid_323_0409_nogap/323_0409_steady/323_0409_steady/323_0409_steady_323_0411_STEADY/323_0409_steady_323_0411_STEADY.g',g)
    # tecfile.write('/home/yxs/F/learning/CFL3D/323_0409/numeca_0409_323_steady/new_grid_323_0409_nogap/323_0409_steady/323_0409_steady/323_0409_steady_323_0411_STEADY/323_0409_steady_323_0411_STEADY.q',q)



import pexpect
import paramiko
import os
#在e2节点编译CFL3D
def run():
    command_source = 'rsync -r /home/yxs/F/learning/CFL3D/CFL3D/source public@e2:/home/public/yxs/CFL3D/'
    os.system(command_source)
if __name__=='__main__':
    run()


from pathlib import Path
import numpy as np 
from . import one2one
import time
from yxspkg import tecfile
import math
import re
from . import functions
from . import input_cfl3d
#该模块主要是用来处理轴流式旋转机械的，并且直接可产生cfl3d可用的inp文件进行计算


def RotorStatorCal(blocks,Stator=True,res = 1e-3):
    #find IN OUT
    blocks_info = one2one.generate_blocks_info(blocks)
    blocks_faces = one2one.generate_blocks_face(blocks)
    face_z_list = [(i,key,(value[2][0]+value[2][1])/2) for i,block in enumerate(blocks_info) for key,value in block.items()]
    face_z_list.sort(key = lambda x:x[2])
    res = abs(face_z_list[0][2] - face_z_list[-1][2])*0.00001
    t = face_z_list[0][2]
    face_in_list = [i for i in face_z_list if abs(i[2]-t) < res]
    t = face_z_list[-1][2]
    face_out_list = [i for i in face_z_list if abs(i[2]-t) < res]

    used_faces = [(i,key) for i,key,_ in face_in_list+face_out_list]

    result = {'in':face_in_list,'out':face_out_list}



    face_top_list = []
    face_bottom_list = []
    for i,faces in enumerate(blocks_faces):
        for key1,key2 in (('K0','KE'),('J0','JE'),('I0','IE')):
            if (i,key1) in used_faces or (i,key2) in used_faces:
                continue
            face1x,face1y,_ = faces[key1]
            face2x,face2y,_ = faces[key2]
            d1 = np.sqrt(face1x**2 + face1y**2)
            d2 = np.sqrt(face2x**2 + face2y**2)
            d = d1-d2 
            dmin,dmax = d.min(),d.max()
            if dmin>res:
                face_top_list.append((i,key1,dmin))
                face_bottom_list.append((i,key2,dmin))
            if dmax<-res:
                face_top_list.append((i,key2,dmax))
                face_bottom_list.append((i,key1,dmax))
    result['top'] = face_top_list
    result['bottom'] = face_bottom_list

    used_faces.extend([(i,k) for i,k,d in face_top_list+face_bottom_list])
    # for i in used_faces:
    #     print(i)
    x0 = max([j[0][1] for i in blocks_info for j in i.values()])
    y0 = max([j[1][1] for i in blocks_info for j in i.values()])
    xy0norm = (x0**2+y0**2)**0.5
    amax, amin = None,None
    for i, faces in enumerate(blocks_faces):
        for key,face in faces.items():
            if (i,key) in used_faces:
                continue
            x1,y1,_ = face
            fz = x0*x1+y0*y1 
            fm = np.power(x1**2+y1**2,1/2)
            angle = fz/(fm*xy0norm)
            anglemax = angle.max()
            anglemin = angle.min()
            if amax is None or anglemax > amax:
                amax = anglemax 
                face_max = i,key,amax
            if amin is None or anglemin < amin:
                amin = anglemin
                face_min = i,key,amin
    result['periodic'] = [face_max,face_min]
    #寻找叶片壁面
    in_out_id = set([i[0] for i in result['in']+result['out']])
    ijk_to_axis = {'i':0,'j':1,'k':2}
    axis_to_ijk = 'IJK'
    for blockid,block in enumerate(blocks):
        if blockid in in_out_id:continue
        Ogrid = one2one.Ogrid_check(block)
        if Ogrid:
            _,_,_,index1,index2 = Ogrid[0]
            axis = 6 - index1 - index2 -1
            for bottom in result['bottom']:
                if bottom[0] == blockid:
                    bottom_face = bottom[1]
            axis2 = ijk_to_axis[bottom_face[0].lower()]
            axis3 = 3 - axis - axis2
            x,y,z = block['X'],block['Y'],block['Z']
            sl = [0,0,0] 
            sl[axis] = slice(0,None)
            sl1 = tuple(sl)
            x1,y1,z1 = x[sl1],y[sl1],z[sl1]
            sl[axis3] = -1
            sl2 = tuple(sl)
            x2,y2,z2 = x[sl2],y[sl2],z[sl2]
            v1 = (x1.max()-x1.min())*(y1.max()-y1.min())*(z1.max()-z1.min())
            v2 = (x2.max()-x2.min())*(y2.max()-y2.min())*(z2.max()-z2.min())
            if v1<v2:
                key = '0'
            else:
                key = 'E'
            faceId = axis_to_ijk[axis3]+key 
            result['bottom'].append((blockid,faceId, v2-v1))
    return result
def CalDeltaAngle(line):
    px,py,pz = line
    max_angle = 0
    max_number = -1
    for i in range(1,len(px)-1):
        x1,y1,z1 = px[i-1],py[i-1],pz[i-1]
        x2,y2,z2 = px[i],py[i],pz[i]
        x3,y3,z3 = px[i+1],py[i+1],pz[i+1]
        dx1,dy1,dz1 = x2-x1,y2-y1,z2-z1
        dx2,dy2,dz2 = x3-x2,y3-y2,z3-z2

        mul = dx1*dx2+dy1*dy2+dz1*dz2 
        norm = ((dx1**2+dy1**2+dz1**2)*(dx2**2+dy2**2+dz2**2))**0.5
        cosv = mul/norm 
        if cosv>1:cosv = 1
        if cosv<-1:cosv = -1
        angle = math.acos(cosv)
        if angle > max_angle:
            max_angle = angle 
            max_number = i 
    return max_number,max_angle/3.1415*180
def SplitBlock(blocks,splicted,init_q):
    bl = blocks[splicted]
    x,y,z = bl['X'],bl['Y'],bl['Z']
    max_axis,max_angle,max_number = -1,0,0
    for i in range(3):
        for j in (0,-1):
            if i == 0:
                line = x[:,j,j],y[:,j,j],z[:,j,j]
            elif i == 1:
                line = x[j,:,j],y[j,:,j],z[j,:,j]
            else:
                line = x[j,j,:],y[j,j,:],z[j,j,:]
            max_n,max_a = CalDeltaAngle(line)
            if max_a > max_angle:
                max_angle = max_a
                max_axis = i
                max_number = max_n
    sl = [slice(0,None)]*3
    sl[max_axis] = slice(0,max_number+1)
    sl1 = tuple(sl)
    bl1 = {'X':x[sl1],'Y':y[sl1],'Z':z[sl1]}
    sl[max_axis] = slice(max_number,None)
    sl2 = tuple(sl)
    bl2 = {'X':x[sl2],'Y':y[sl2],'Z':z[sl2]}
    block_list = blocks[:splicted] + [bl1,bl2] + blocks[splicted+1:]
    if init_q:
        ql = init_q[splicted]
        ql1,ql2 = dict(),dict()
        for key,val in ql.items():
            if isinstance(key,str):
                ql1[key] = val 
                ql2[key] = val 
            else:
                ql1[key] = val[sl1]
                ql2[key] = val[sl2]
        init_q[splicted] = ql2
        init_q.insert(splicted,ql1)
    return block_list

def AutoGenerateInputDdata(blocks,stators=None,rotors=None,split_blocks=None,rpm = 10,merged = True,bc2006=True,**karg):
    blocks_old = blocks 
    split_blocks.sort()
    split_blocks = list(split_blocks)
    split_blocks_old = tuple(split_blocks)
    stators_old, rotors_old = tuple(stators), tuple(rotors)
    def after_split(splitid,stage):
        t = [(i+1 if i>splitid else i) for i in stage]
        if splitid in t:
            t.append(splitid+1)
        t.sort()
        return t
    init_file = karg.get('init_file')
    if init_file:
        init_q = tecfile.read(init_file)
        assert len(init_q) == len(blocks)
        for i,j in zip(blocks,init_q):
            toshape = i['X'].shape
            for key,val in j.items():
                if isinstance(key,str):
                    continue
                todata = functions.interpolateblock(val,toshape)
                j[key] = todata
    else:
        init_q = None
    if split_blocks is not None:
        for i in range(len(split_blocks)):
            splictid = split_blocks[i]
            if not merged:
                init_q_split = init_q 
            else:
                init_q_split = None
            blocks = SplitBlock(blocks,splictid,init_q_split)
            for j in range(i+1,len(split_blocks)):
                split_blocks[j] += 1
            stators = [after_split(splictid,i) for i in stators]
            rotors  = [after_split(splictid,i) for i in rotors]
    if init_file:
        for q in init_q:
            for key,val in q.items():
                if isinstance(key,str):
                    continue 
                center = (val[1:,1:,1:]+
                         val[1:,:-1,1:]+
                         val[1:,1:,:-1]+
                         val[1:,:-1,:-1]+
                         val[:-1,1:,1:]+
                         val[:-1,:-1,1:]+
                         val[:-1,1:,:-1]+
                         val[:-1,:-1,:-1])/8
                if center.dtype != np.float32:
                    center = center.astype('float32')
                q[key] = center
        init_path =str( Path(karg['workdir']) / 'init.q')
        tecfile.write(init_path, init_q)
    boundary_stators = []
    for stator in stators:
        blocks_stator = [blocks[i] for i in stator]
        m = RotorStatorCal(blocks_stator,Stator=True)
        t = {k:[(stator[i[0]],i[1],i[2]) for i in v] for k,v in m.items()}
        boundary_stators.append((min(stator),t))
    boundary_rotors = []
    for rotor in rotors:
        blocks_rotor = [blocks[i] for i in rotor]
        m = RotorStatorCal(blocks_rotor,Stator=True)
        t = {k:[(rotor[i[0]],i[1],i[2]) for i in v] for k,v in m.items()}
        boundary_rotors.append((min(rotor),t))
   
    periodic_faces = [(i['periodic'][0][:2],i['periodic'][1][:2],True) for _,i in boundary_stators+boundary_rotors]
    periodic_result = list()
    one2oneData = one2one.one2one(blocks,periodic_faces,periodic_faces_result=periodic_result)

    assert len(periodic_faces) == len(periodic_result)

    one2oneData = [i for i in one2oneData if i not in periodic_result]
    
    rotors_id = [i[0] for i in boundary_rotors]

    boundary_conditions = boundary_stators + boundary_rotors
    boundary_conditions.sort(key = lambda x:x[0])
    boundary_result = {}
    for (f1,f2,_),(_,one1,_,one2) in zip(periodic_faces,periodic_result):
        bl1,key1 = f1
        bl2,key2 = f2
        face1 = one2one.GetFace(blocks[bl1], key1)
        face2 = one2one.GetFace(blocks[bl2], key2)
        p0 = face1[0][0,0],face1[1][0,0]
        angle1 = one2one.GetFaceAngle(p0,face1)
        angle2 = one2one.GetFaceAngle(p0,face2)
        delta = angle2[-1] - angle1[-1]
        delta *= -180/math.pi
        key = '{}{}'.format(bl1+1,key1)
        boundary_result[key] = {'bctype':2005,'one2one':one1,'auxdata':(bl2+1,0,0,delta)}
        key = '{}{}'.format(bl2+1,key2)
        boundary_result[key] = {'bctype':2005,'one2one':one2,'auxdata':(bl1+1,0,0,-delta)}
    for rs_id,condition in boundary_conditions:
        if rs_id in rotors_id:
            rotated = 1
        else:
            rotated = -1
        for i in condition['bottom']:
            key = '{}{}'.format(i[0]+1,i[1])
            boundary_result[key]={'bctype':2004}
        for i in condition['top']:
            key = '{}{}'.format(i[0]+1,i[1])
            boundary_result[key]={'bctype':-rotated*2004}
    for i in boundary_conditions[0][1]['in']:
        key = '{}{}'.format(i[0]+1,i[1])
        boundary_result[key] = {'bctype':2003,'data':karg['in_condition']}

    top_face_dict = {i[0]:i[1] for i in boundary_conditions[-1][1]['top']}
    IJK_123 = dict(zip("IJK",(1,2,3)))
    if bc2006:
        for i in boundary_conditions[-1][1]['out']:
            key = '{}{}'.format(i[0]+1,i[1])
            faceId = top_face_dict[i[0]]
            intdir = IJK_123[faceId[0]]
            if faceId[-1] == '0':
                intdir *= -1
            boundary_result[key] = {'bctype':2006,'data':(0,0,intdir,3)}
    else:
        for i in boundary_conditions[-1][1]['out']:
            key = '{}{}'.format(i[0]+1,i[1])
            
            boundary_result[key] = {'bctype':2002}
    
    patchedData = list()
    steady = karg.get('steady',False)
    # periodic_result_block = set([j for i in periodic_result for j in (i[0]-1,i[2]-1)])
    for i in range(len(boundary_conditions) - 1):
        rs_id1,condition1 = boundary_conditions[i]
        rs_id2,condition2 = boundary_conditions[i+1]
        face1 = [i[:2] for i in condition1['out']]
        face2 = [i[:2] for i in condition2['in']]
        if rs_id1 in rotors_id:
            face1,face2 = face2,face1
        rotate_angle = 'positive'
        if rpm < 0:
            rotate_angle = 'negative'
        t = [one2one.patchedFace2Faces(blocks,ff1,face2,'rotor',rotate_angle,steady) for ff1 in face1]
        patchedData.extend(t)
        t = [one2one.patchedFace2Faces(blocks,ff2,face1,'stator',rotate_angle,steady) for ff2 in face2]
        patchedData.extend(t)

    # calculate rotated_degree
    if rotors:
        t = [abs(k[-1]) for tof,fromf in patchedData for k in fromf if abs(k[-1])>0.000001 and k[0][0] == rotors_id[0]]
        rotated_degree = min(t)/3.1415926*180

    if merged:
        for i in reversed(split_blocks):
            blocks, one2oneData, patchedData, boundary_result = one2one.MergeBlocks(blocks,i,i+1,one2oneData,patchedData,boundary_result)
    if merged:
        nrotors = rotors_old
    else:
        nrotors = rotors
    if rotors_old and steady is False:
        rotated_dict = {'axis':(0,0,1),'max':rotated_degree, 'rpm':rpm}
        rotatedBlocks = {i+1:rotated_dict for rotor in nrotors for i in rotor}
    else:
        rotatedBlocks = None
    #check oneonedata
    if merged:
        multi = stators_old + rotors_old
    else:
        multi = stators + rotors
    # for i in multi:
    #     print(i)
    for ii,(bl1,_,bl2,_) in enumerate(one2oneData):
        for sr in multi:
            if bl1-1 in sr:
                if bl2-1 in sr:
                    break 
                else:
                    print('warning',ii,bl1,bl2)
        else:
            print(ii,bl1,bl2)
            raise Exception('oneonedata error')
    return blocks,one2oneData,patchedData,boundary_result,rotatedBlocks

def set_inputfile(workdir,input_filename,mesh_file,LREF,stators=None,rotors=None,split_blocks = None,
                 merged=True,periodic_steps = 200,rpm = 0,number_of_time_step=2000,
                 CFL_number=5,xmach=None,env_viscosity=1.716e-5,env_temperature = None,
                 turbulence_model=5,
                 output_pressure = 106000,
                 **kargs):  

    bs = tecfile.read(mesh_file)
    bs = one2one.AdjustNegativeGrid(bs)
    Pt,Tt = kargs['total_pressure'] , kargs['total_temperature'] 
    t = AutoGenerateInputDdata(bs,stators=stators,rotors=rotors,
             split_blocks=split_blocks,rpm =rpm,merged=merged,bc2006 = kargs.get('bc2006',True),
             in_condition = (Pt,Tt),init_file = kargs.get('init_file'),workdir=workdir,steady=kargs.get('steady',False))
    bs,one2oneData,patchedData,boundary_result,rotatedBlocks = t
    if rotatedBlocks:
        keys = list(rotatedBlocks.keys())
        v = rotatedBlocks[keys[0]]['max']
        v /= periodic_steps
        t = rpm / 60
        dtime = v / (t*360)
        print('time step',dtime)
    else:
        dtime = 1e-5
    if kargs.get('steady'):
        number_of_time_step = 1
    IUNST = 1
    init_file = kargs.get('init_file')
    if init_file:
        init_file = 'init.q'
    a = input_cfl3d.inputFile(
        workdir=workdir,
        gridfile = bs,
        input_filename=input_filename,
        LREF = LREF,
        CFL_number=CFL_number,
        xmach = xmach,
        env_viscosity=env_viscosity,
        env_temperature = kargs['total_temperature'] ,
        env_pressure = kargs['total_pressure'],
        env_gamma = kargs['env_gamma'],
        turbulence_model=5,
        output_pressure = output_pressure,
        I2D = kargs.get('I2D',0),IUNST=kargs.get('IUNST',IUNST),MOVIE=kargs.get('MOVIE',0),
        rotated_center=kargs.get('rotated_center',True),
        NWREST=kargs.get('NWREST',100),IREST=kargs.get('IREST',0),
        number_of_time_step=number_of_time_step,time_step = dtime,
        BOUNDARY = boundary_result,
        oneOne = one2oneData,
        patchedData = patchedData,ITOSS = kargs.get('ITOSS',0),ITMAX = kargs.get('ITMAX',None),
        rotatedBlocks = rotatedBlocks,
        IPTYPE = kargs.get('IPTYPE',1),
        NCYC = kargs.get('NCYC',20),
        ITA = kargs.get('ITA',-2),
        NCG = kargs.get('NCG',2),
        restart_file = kargs.get('restart_file',None),
        IYXS = kargs.get('IYXS'),
        init_file = init_file,
        IALPH = kargs.get('IALPH',1)

    )
    a.start_write()
if __name__ == '__main__':
    g1 = '/home/yxs/F/pythonAPP/cfl3d_pyui/RS_work/rotate323_mesh.g'
    g2 = '/home/yxs/F/learning/CFL3D/323_0409/new_grid_323_0409_nogap/323_0409.g'
    g3 = '/home/yxs/F/learning/CFL3D/323_0409/ok/new_grid_323_0409_nogap/323_0409.g'
    set_inputfile(
        workdir='./test3',
        input_filename = 'rotate323_0409.inp',
        mesh_file = g3,
        LREF = 1.0/1000,
        stators=[range(15),range(15,25)],
        rotors=[range(25,40)],
        split_blocks=[19,24],
        rpm =15000,merged=False,periodic_steps=200,
        number_of_time_step=2000,
        MOVIE = 100,  #每多少步自动保存
        CFL_number=3,  
        xmach = 0.4,
        env_viscosity=1.716e-5,
        env_gamma = 1.4,
        total_pressure = 101325,
        total_temperature = 288.15,
        output_pressure = 106000,
        turbulence_model=5,
        IPTYPE = 0,
        NCYC = 20,
        ITA = -2,  #时间采用二阶精度
        NCG = 2,  #多重网格
        rotated_center=True,
        bc2006=False,
        IYXS = 3,
        IALPH=0,
        steady=False
        # init_file = '/home/yxs/F/learning/CFL3D/323_0409/numeca_0409_323_steady/new_grid_323_0409_nogap/323_0409_steady/323_0409_steady/323_0409_steady_323_0411_STEADY/323_0409_steady_323_0411_STEADY.q'
        # restart_file = '/home/yxs/F/learning/CFL3D/323_304/323_304_e1/rotate323_restart.bin',
    )


import os
import shutil
from pathlib import Path 
import sys
import time
def compile_(mpi = False):
    source_cfl3d = Path('/home/yxs/F/learning/CFL3D/CFL3D/source/cfl3d')
    dist = source_cfl3d / 'dist'
    libs = source_cfl3d / 'libs'
    fortran_F = list(dist.glob('*.F')) + list(libs.glob('*.F')) + list(libs.glob('*.F90')) + list(dist.glob('*.F90'))

    F_time = {i.stem:(i,i.lstat().st_mtime) for i in fortran_F}


    assert len(F_time) == len(fortran_F)
    if mpi:
        seq_source = source_cfl3d.parent.parent / 'build' / 'cfl' / 'mpi'
    else:
        seq_source = source_cfl3d.parent.parent / 'build' / 'cfl' / 'seq'
    libs_source = source_cfl3d.parent.parent / 'build' / 'cfl' / 'libs'
    O_time_list = list(seq_source.glob('*.o')) + list(libs_source.glob('*.o'))
    O_time = {i.stem:(i,i.lstat().st_mtime) for i in O_time_list}

    # assert len(O_time) == len(O_time_list)
    delete_files = []
    old_time = 0
    for key,value in O_time.items():
        Opath, Omtime = value 
        value_F = F_time[key]
        Fpath, Fmtime = value_F
        if Fmtime > Omtime:
            os.remove(Opath)
            if Fmtime > old_time:
                old_time = Fmtime
            print("Delete ",Opath)
            delete_files.append(str(Fpath))

    for i in delete_files:
        rsync_file(i)
        print("同步到e2",i)
    build_path = source_cfl3d.parent.parent / 'build'
    os.chdir(build_path)
    print('当前目录',os.getcwd())
    if mpi:
        os.system('make cfl3d_mpi')
        p = build_path / 'cfl' /'mpi'/'cfl3d_mpi'
    else:
        os.system('make cfl3d_seq')
        p = build_path / 'cfl' /'seq'/'cfl3d_seq'
    new_time = p.lstat().st_mtime 
    if new_time < old_time or old_time == 0:
        input("本次编译对执行文件没有产生任何变化，是否继续[y]")
    
    return p 
def exit_mpi():
    k = os.popen('ps -A | grep mpi').read()
    cm = k.split()
    if cm:
        os.system('kill {}'.format(cm[0]))
def rsync_file(filename):
    n = filename.find('CFL3D/source')
    if n != -1:
        dest = filename[n:]
    else:
        return
    command = f'rsync {filename} public@e2:/home/public/yxs/{dest}'
    os.system(command)
if __name__=='__main__':
    arg1 = sys.argv[1]
    # f = Path(sys.argv[1]).absolute()
    # print(f)
    if  arg1.startswith('-'):
        f = Path(sys.argv[2]).absolute()
        print(f)
        p = compile_(True)
        mpif = f.parent / 'cfl3d_mpi'
        if mpif.exists():
            os.remove(mpif)
        shutil.copy(p,f.parent)
        os.chdir(f.parent)
        print('当前目录',os.getcwd())
        print("开始运行")
        os.system('mpirun -np {} ./cfl3d_mpi < {}'.format(int(arg1[1:]),f.name))
        try:
            time.sleep(365*24*60*60)
        except:
            print('exit mpi')
            exit_mpi()
    else:
    
        f = Path(arg1).absolute()
        print(f)
        p = compile_()
        shutil.copy(p,f.parent)
        os.chdir(f.parent)
        print('当前目录',os.getcwd())
        print("开始运行")
        os.system('./cfl3d_seq < {}'.format(f.name))


from yxspkg import plot3d
from pathlib import Path
import os
import shutil
import math
from . import one2one as o2o
class inputFile:
    def __init__(self,workdir,gridfile,timestep=1,turbulence_model=5,ALPHA=90,BETA=0,
                input_filename = 'input.inp',keyword_driven=False,work_name="CFL3D Case configuration",
                **kargs):
        #环境温度 默认 300K
        self.env_temperature = kargs['env_temperature']
        #环境压力 默认 101000Pa
        self.env_pressure = kargs['env_pressure']
        #介质比热比 默认空气 1.4
        self.env_gamma = kargs['env_gamma']
        #气体常数R= 287.14
        self.env_R = kargs.get('env_R',287.14)
        #入口速度  单位m/s 
        self.xmach = kargs['xmach']
        #动力年度 应力与应变速率之比 国际单位 Pa.s
        self.env_viscosity = kargs['env_viscosity']
        #LREF 相当于fluent中设置单位的选项
        self.LREF = kargs['LREF']
        if abs(self.LREF - 1) > 1e-9:
            self.scale_mesh = self.LREF 
            self.LREF = 1
        else:
            self.scale_mesh = None
        #计算声速
        self.env_acusitic_velocity = (self.env_gamma*self.env_R*self.env_temperature)**0.5
        #reue
        self.reue = self.env_pressure/(self.env_R*self.env_temperature)*self.xmach*self.env_acusitic_velocity / self.env_viscosity*1e-6
        #攻角
        self.envAlpha = ALPHA
        self.envBeta = BETA
        #CFL数
        self.CFL_number = kargs.get('CFL_number',5)
        #时间步数
        self.number_of_time_step = kargs.get('number_of_time_step',1)
        #时间步长
        self.time_step = kargs.get('time_step',0)
        #ita 这个参数表示迭代策略和精度  |ITA|=2 表示二阶时间精度
        self.ITA = kargs.get('ITA',-2)
        #DT
        self.DT = kargs.get('DT',None)
        #IREST 是否继续上次的算列算 
        self.IREST = kargs.get('IREST',0)
        #iflags 时间步是否是可变的
        self.IFLAGTS = kargs.get('IFLAGTS',0)
        #fmax 时间步变化的最大倍数，当iflagts>0时可用
        self.FMAX = kargs.get('FMAX',1)
        #将boundary的faceId都调整为大写
        kargs['BOUNDARY'] = {key.upper():{t.upper():v for t,v in value.items()} for key,value in kargs['BOUNDARY'].items()}

        self.workdir = workdir
        self.pwd = os.getcwd()
        os.chdir(workdir)
        self.gridfile_name = gridfile
        self.timestep = timestep
        self.turbulence_model = turbulence_model
        self.fp = open(input_filename,'w')
        self.input_filename = input_filename
        self.keyword_driven = keyword_driven 
        self.kargs = kargs
        self.work_name = work_name
    def start_write(self):
        #当前目录位于 workdir
        kargs = self.kargs
        fp = self.fp 
        fp.write('I/O FILES\n') #文件开头
        #self.gridfile_name 存储输入文件的名字，self.gridfile 存储将网格放入workdir后的名字
        self.gridfile = self.read_gridfile(self.gridfile_name)
        # p = Path(self.gridfile)
        fp.write(self.gridfile+'\n')
        # fp.write('output.g\noutput.q\n')
        t = self.input_filename[:-4]+'_'
        
        other_files = ['output.g','output.q','cfl3d.out','cfl3d.res','cfl3d.turres','cfl3d.blomax','cfl3d.out15',
                       'cfl3d.prout','cfl3d.out20','ovrlp.bin','patch.bin\n']
        fp.write('\n'.join([t+i for i in other_files]))
        restart_file = self.get_restart_file()
        fp.write(restart_file+'\n')
        self.write_keyword_driven()
        #写入工程名称
        fp.write('  {}\n'.format(self.work_name.strip()))
        self.write_IYXS_block()
        self.write_XMACH_line()
        self.write_SREF_line()
        self.write_DT_line()
        self.write_NGRID_line()
        self.write_NCG_line()
        self.write_IDIM_line()
        #指定层流区
        self.write_ILAMLO_line()
        #嵌入网格 Embedded Mesh Specifications
        self.write_INEWG_line()
        #矩阵求逆和通量限制
        self.write_IDIAG_line()
        #空间差分
        self.write_IFDS_line()
        #边界条件段 和 重叠网格 默认无重叠网格
        self.write_IOVRLP_line()
        #边界条件设置
        self.write_BOUNDARY_block()
        #关于多重网格迭代的设置
        self.write_MSEQ_line()
        #光顺
        self.write_ISSC_line()
        #迭代步数和多重网格的一些设定
        self.write_NCYC_line()
        #粗网格迭代
        self.write_MIT1_line()
        #1-1 block data
        self.write_One_to_One_block()
        #交接面信息
        self.write_NINTER_block()
        #输出数据设置
        self.write_OUTPUT_block()
        #控制面信息，检测某一个面的流量，力，速度的设置 
        self.write_CONTROL_SURFACE_block()
        #平移网格设置
        self.write_TRANS_block()
        #旋转网格的设置
        self.write_ROTATE_block()
        #动网格交界面插值设置
        self.write_DYNAMIC_PATCH_block()
    def get_restart_file(self):
        t = self.input_filename[:-4]+'_'
        restart_file = self.kargs.get('restart_file')
        if restart_file:
            ff = t+'restart.bin'
            shutil.copy(restart_file,ff)
            self.kargs['IREST'] = 1
            return ff
        else:
            return t+'restart.bin'
    def write_keyword_driven(self):
        if self.keyword_driven:
            pass 
        else:
            return None
    def read_gridfile(self,filename):
        if isinstance(filename,str):
            k = plot3d.read(filename)
        else:
            k = filename
        if self.scale_mesh is not None:
            k = [{key:val*self.scale_mesh for key,val in i.items()} for i in k]
        self.blocks = k
        ftype = k[0]['X'].dtype 
        if ftype == 'float32':
            k = [{key:v.astype('float64') for key,v in i.items()} for i in k]
        self.NGRID = -len(k)
        self.blocks_ijk = [i['X'].shape for i in k]
        fname = self.input_filename[:-4]+'_'+'mesh.g'
        plot3d.write(fname,k)
        return fname
    def write_IYXS_block(self):
        if self.kargs.get('IYXS') is None:
            return
        fp = self.fp 
        fp.write('    IYXS\n     {}\n'.format(self.kargs.get('IYXS')))
        init_file = self.kargs.get('init_file')
        if init_file is not None:
            ifile = 2
            if init_file != 'init.q':
                shutil.copy(init_file,'init.q')
                init_file = 'init.q'
        else:
            init_file = 'init.q'
            ifile = 0
        fp.write("     init_file ig yxs_g  iq yxs_q\n")
        fp.write('     {}  {}  0   yxs.g   0  yxs.q\n'.format(ifile,init_file))
        fp.write('     noninflag\n       0\n')
        fp.write('     mean\n       0\n')
        fp.write('     underrelaxed\n       0\n')
    def write_XMACH_line(self):
        title = '     XMACH     ALPHA      BETA  REUE,MIL   TINF,DR     IALPH    IHSTRY\n'
        self.fp.write(title)
        
        self.envMach = self.xmach
        IALPH = self.kargs.get('IALPH',0)
        XMACH = self.xmach 
        ALPHA = self.envAlpha 
        BETA  = self.envBeta
        REUE = self.reue 
        TINF = self.env_temperature*9/5
        IHSTRY = 0
        fstr = '{:>10.4f}'*5+'{:>10d}{:>10d}\n'
        self.fp.write(fstr.format(XMACH,ALPHA,BETA,REUE,TINF,IALPH,IHSTRY))
    def write_SREF_line(self):
        #不太清楚这一栏是干嘛的 先写成这样的值吧
        title ='      SREF      CREF      BREF       XMC       YMC       ZMC\n'
        value ='   1.00000   1.00000    1.0000    0.00000   0.0000    0.0000\n'
        self.fp.write(title)
        self.fp.write(value)
    def write_DT_line(self):
        kargs = self.kargs
        title = '        DT     IREST   IFLAGTS      FMAX     IUNST   CFL_TAU\n'
        self.fp.write(title)
        DT = kargs.get('DT')
        if self.number_of_time_step == 1:
            DT = str(-self.CFL_number)
        if DT is None:
            
            DT = self.time_step*self.env_acusitic_velocity
            DT /= self.LREF
            DT = '+{}'.format(DT)[:7]
        else:
            DT = str(DT)
        #IREST 是否继续上次的算列算 
        IREST = kargs.get('IREST',0)
        #iflags 时间步是否是可变的
        IFLAGTS = kargs.get('IFLAGTS',0)
        #fmax 时间步变化的最大倍数，当iflagts>0时可用
        FMAX = kargs.get('FMAX',1)
        #iunst 网格是否是定常的
        IUNST = kargs.get('IUNST',0)
        #CFL数的大小，当ITA<0时
        CFL_TAU = kargs.get('CFL_TAU')
        if float(DT) < 0:
            CFL_TAU = 0
        if CFL_TAU is None:
            if self.ITA < 0:
                CFL_TAU = self.CFL_number
        fstr = '{:>10s}{:>10d}{:>10d}{:>10.6f}{:>10d}{:>10.6f}\n'
        self.fp.write(fstr.format(DT,IREST,IFLAGTS,FMAX,IUNST,CFL_TAU))
    def write_NGRID_line(self):
        title = '     NGRID   NPLOT3D    NPRINT    NWREST      ICHK       I2D    NTSTEP       ITA\n'
        self.fp.write(title)
        NGRID = self.NGRID
        NPLOT3D = abs(NGRID)
        NWREST = self.kargs.get('NWREST',1000000)
        NPRINT = self.kargs.get("NPRINT",0)
        ICHK = self.kargs.get('ICHK',0)
        I2D = self.kargs['I2D']
        NTSTEP = self.kargs.get('NTSTEP',self.number_of_time_step)
        ITA = self.kargs.get('ITA',-2)
        fstr = '{:>10d}'*8+'\n'
        self.fp.write(fstr.format(NGRID,NPLOT3D,NPRINT,NWREST,ICHK,I2D,NTSTEP,ITA))
    def write_NCG_line(self):
        title = '       NCG       IEM  IADVANCE    IFORCE  IVISC(I)  IVISC(J)  IVISC(K)\n'
        self.fp.write(title)
        NCG = self.kargs.get('NCG',0)
        self.multigrid = NCG
        IEM = 0
        IADVANCE = 0
        IFORCE = 0
        fstr = '{:>10d}'*7+'\n'
        m = self.turbulence_model
        if self.kargs.get('I2D',1) == 1:
            m2d = 0 
        else:
            m2d = m
        for _ in range(abs(self.NGRID)):
            self.fp.write(fstr.format(NCG,IEM,IADVANCE,IFORCE,m2d,m,m))
    def write_IDIM_line(self):
        title = '      IDIM      JDIM      KDIM\n'
        self.fp.write(title)
        fstr = '{:>10d}'*3+'\n'
        for i,j,k in self.blocks_ijk:
            self.fp.write(fstr.format(i,j,k))
    def write_ILAMLO_line(self):
        title = '    ILAMLO    ILAMHI    JLAMLO    JLAMHI    KLAMLO    KLAMHI\n'
        self.fp.write(title)
        fstr = '{:>10d}'*6+'\n'
        for _ in range(abs(self.NGRID)):
            self.fp.write(fstr.format(0,0,0,0,0,0))
    def write_INEWG_line(self):
        title = '     INEWG    IGRIDC        IS        JS        KS        IE        JE        KE\n'
        self.fp.write(title)
        fstr = '{:>10d}'*8+'\n'
        for _ in range(abs(self.NGRID)):
            self.fp.write(fstr.format(0,0,0,0,0,0,0,0))
    def write_IDIAG_line(self):
        #矩阵求逆 和 通量限制
        
        title = '  IDIAG(I)  IDIAG(J)  IDIAG(K)  IFLIM(I)  IFLIM(J)  IFLIM(K)\n'
        self.fp.write(title)
        fstr = '{:>10d}'*6+'\n'
        for _ in range(abs(self.NGRID)):
            self.fp.write(fstr.format(1,1,1,0,0,0))
    def write_IFDS_line(self):
        #空间差分
        self.fp.write('   IFDS(I)   IFDS(J)   IFDS(K)  RKAP0(I)  RKAP0(J)  RKAP0(K)\n')
        fstr = '{:>10d}'*3+'{:>10.5f}'*3+'\n'
        for _ in range(abs(self.NGRID)):
            self.fp.write(fstr.format(1,1,1,1/3,1/3,1/3))
    def write_IOVRLP_line(self):
        #边界条件段和重叠网格 默认无重叠网格
        self.fp.write('      GRID     NBCI0   NBCIDIM     NBCJ0   NBCJDIM     NBCK0   NBCKDIM   IOVRLP \n')
        fstr = '{:>10d}'*8+'\n'
        for i in range(abs(self.NGRID)):
            self.fp.write(fstr.format(i+1,1,1,1,1,1,1,0))

    def SetBoundaryLine(self,faceId):
        #faceId:Nmn  N:blockid,m:I or J or K, n:0 or E or e
        fstr = '{:>10d}'*8+'\n'
        blockId = int(faceId[:-2])
        boundary = self.kargs['BOUNDARY'].get(faceId)
        if not boundary:
            return fstr.format(blockId,1,0,0,0,0,0,0)
        bctype = boundary['BCTYPE']
        if bctype == 0:
            result = fstr.format(blockId,1,0,0,0,0,0,0)
        elif abs(bctype) == 2004:#无滑移
            result = fstr.format(blockId,1,bctype,0,0,0,0,2)
            result += 'Tw      Cq\n0.      0.\n'
        elif bctype == 2003:#入口条件
            result = fstr.format(blockId,1,2003,0,0,0,0,5)
            result +='Mach  Pt/Pinf  Tt/Tinf  alpha  beta\n'
            inFstr = '{:5.4f}{:>8.5f} {:>9.5f} {:>7.5f} {:>6.5f}\n'
            Pt,Tt = boundary['DATA']
            Tt/=self.env_temperature 
            Pt/=self.env_pressure
            result += inFstr.format(self.envMach,Pt,Tt,self.envAlpha,self.envBeta)
        elif bctype == 2002:#压力出口条件
            result = fstr.format(blockId,1,bctype,0,0,0,0,1)
            pressure = boundary.get('PRESSURE')
            if pressure is None:
                pressure = self.kargs['output_pressure']
            result += 'pexit/pinf\n{:.5f}\n'.format(pressure/self.env_pressure)
        elif bctype == 2006:#适合叶轮机械的压力径向平衡的出口条件
            result = fstr.format(blockId,1,bctype,0,0,0,0,4)
            ngridc,p_p,intdir,axcoord = boundary['DATA']
            if p_p == 0:
                p_p = self.kargs['output_pressure'] / self.env_pressure
            result += 'ngridc  P/Pinf  intdir axcoord\n{:d} {:.4f} {:+4d} {:4d}\n'.format(
                ngridc,p_p,intdir,axcoord)
        elif bctype == 2005:
            t = [(i,j) for i,j in boundary['ONE2ONE'][:3] if i!=j]
            t = [i for v in t for i in v]
            result = fstr.format(blockId,1,bctype,*t,4)
            result += '    ngridp      dthx      dthy      dthz\n{:d} {:.6f} {:6f} {:6f}\n'.format(*boundary['AUXDATA'])
        else:
            raise Exception('bctype error!!')
        return result
        


    def write_BOUNDARY_block(self):
        title = '{:<6s}GRID   SEGMENT    BCTYPE      JSTA      JEND      KSTA      KEND    NDATA\n'
        
        for i in ['I0:','IDIM:','J0:','JDIM:','K0:','KDIM:']:
            self.fp.write(title.format(i))
            for j in range(abs(self.NGRID)):
                faceId = '{}{}'.format(j+1,i[:2].replace('D','E'))
                boundLine = self.SetBoundaryLine(faceId)
                self.fp.write(boundLine)
    def write_MSEQ_line(self):
        title = '      MSEQ    MGFLAG    ICONSF       MTT      NGAM\n'
        fstr='{:>10d}'*5+'\n'
        #MSEQ:多重网格计算过程中的排序方式,这个参数会涉及到restat文件的保存内容 （目前理解可能存在偏差）,默认参数1
        MSEQ = self.kargs.get('MSEQ',1)
        #MGFLAG：是否采用多重网格的标志，默认不采用多重网格
        MGFLAG = self.kargs.get('MGFLAG',0)
        if self.multigrid is not 0 and MGFLAG is 0:
            MGFLAG = 1
        #NGAM：多重网格迭代方式 V W 重叠网格不推荐W型,默认采用V型
        NGAM = self.kargs.get('NGAM',1)
        #ICONSF 关于嵌入网格通量计算时的设置，默认取1
        ICONSF = self.kargs.get('ICONSF',1)
        #官方推荐0
        MTT = self.kargs.get('MTT',0)
        self.fp.write(title)
        self.fp.write(fstr.format(MSEQ,MGFLAG,ICONSF,MTT,NGAM))
    def write_ISSC_line(self):
        title = '      ISSC  EPSSC(1)  EPSSC(2)  EPSSC(3)      ISSR  EPSSR(1)  EPSSR(2)  EPSSR(3)\n'
        fstr=('{:>10d}'+'{:>10.4f}'*3)*2+'\n'
        self.fp.write(title)
        #
        ISSC = self.kargs.get('ISSC',0)
        #
        EPSSC1 = self.kargs.get('EPSSC1',1/3)
        EPSSC2 = self.kargs.get('EPSSC2',1/3)
        EPSSC3 = self.kargs.get('EPSSC3',1/3)
        #
        ISSR = self.kargs.get('ISSR',0)
        EPSSR1 = self.kargs.get('EPSSR1',1/3)
        EPSSR2 = self.kargs.get('EPSSR2',1/3)
        EPSSR3 = self.kargs.get('EPSSR3',1/3)
        self.fp.write(fstr.format(ISSC,EPSSC1,EPSSC2,EPSSC3,ISSR,EPSSR1,EPSSR2,EPSSR3))
    def write_NCYC_line(self):
        title = '      NCYC    MGLEVG     NEMGL     NITFO\n'
        fstr = '{:>10d}'*4+'\n'

        NCYC = self.kargs.get('NCYC',10)
        MGLEVG = self.kargs.get('MGLEVG',1)
        if self.multigrid is not 0 and MGLEVG is 1:
            MGLEVG = 2
        NEMGL = self.kargs.get('NEMGL',0)
        NITFO = self.kargs.get('NITFO',0)
        self.fp.write(title)
        self.fp.write(fstr.format(NCYC,MGLEVG,NEMGL,NITFO))
    def write_MIT1_line(self):
        title = '      MIT1      MIT2      MIT3      MIT4      MIT5\n'
        fstr = '{:>10d}'*5+'\n'
        self.fp.write(title)
        self.fp.write(fstr.format(1,1,1,1,1))
    def write_One_to_One_block(self):
        oneone_block = self.kargs.get('oneOne','1-1 blocking data:\n')
        if not isinstance(oneone_block,str):
            oneone_block = o2o.transfer_one2one_str(oneone_block)
        self.fp.write(oneone_block.rstrip())
        self.fp.write('\n')
    def write_NINTER_block(self):
        title = '  PATCH SURFACE DATA:\n    NINTER\n'
        self.fp.write(title)
        NINTER = self.kargs.get('NINTER',0)
        assert NINTER == 0
        self.fp.write('{:>10d}\n'.format(0))
    def write_OUTPUT_block(self):
        title = '  PLOT3D OUTPUT:\n   GRID IPTYPE ISTART   IEND   IINC JSTART   JEND   JINC KSTART   KEND   KINC\n'
        self.fp.write(title)
        IPTYPE = self.kargs.get('IPTYPE',0)
        MOVIE = self.kargs.get('MOVIE',0)
        fstr = '{:>7d}'*11+'\n'
        for i in range(abs(self.NGRID)):
            self.fp.write(fstr.format(i+1,IPTYPE,0,0,0,0,0,0,0,0,0))
        self.fp.write('{:>7s}\n{:>7d}\n'.format('MOVIE',MOVIE))

        #PRINT OUT
        title2 = '  PRINT OUT:\n  BLOCK IPTYPE ISTART   IEND   IINC JSTART   JEND   JINC KSTART   KEND   KINC\n'
        self.fp.write(title2)
    def write_CONTROL_SURFACE_block(self):
        title = ' CONTROL SURFACES:\n   NCS\n'
        self.fp.write(title)
        NCS = self.kargs.get('NCS',0)
        assert NCS == 0
        self.fp.write('{:>6d}\n  GRID   ISTA  IEND  JSTA  JEND  KSTA  KEND IWALL INORM\n'.format(NCS))
    def write_TRANS_block(self):
        #动网格设置
        title = '  MOVING GRID DATA - TRANSLATION\n  NTRANS\n{:>8d}\n'
        NTRANS = self.kargs.get('NTRANS',0)
        self.fp.write(title.format(NTRANS))
        if NTRANS != 0:
            pass 
        else:
            tt = '    LREF\n    GRID ITRANS   RFREQ   UTRANS   VTRANS   WTRANS\n    GRID  DXMAX   DYMAX    DZMAX\n'
            self.fp.write(tt)

    def write_ROTATE_block(self):
        #旋转网格设置
        title = '  MOVING GRID DATA - ROTATION\n  NROTAT\n{:>8d}\n'
        rotatedBlocks = self.kargs.get('rotatedBlocks',None)
        rotated_center = self.kargs.get('rotated_center',True)
        if rotated_center:
            rcenter = 1 
        else:
            rcenter = 0
        if rotatedBlocks is not None:
            NROTAT = len(rotatedBlocks)+rcenter 
        else:
            NROTAT = 0
        self.fp.write(title.format(NROTAT))
    
        if NROTAT != 0:
            rotatedBlocks = list(rotatedBlocks.items())
            rotatedBlocks.sort(key=lambda x:x[0])
            LREF = 1/self.LREF
            IROTAT = self.kargs.get('IROTAT',1)
            assert IROTAT == 1
            tt = '    LREF\n    GRID IROTAT   RFREQ   OMEGAX   OMEGAY   OMEGAZ   XORIG   YORIG   ZORIG\n    GRID   THXMAX   THYMAX   THZMAX\n'
            self.fp.write('    LREF\n{:>8.3f}\n'.format(LREF))
            self.fp.write('    GRID IROTAT   RFREQ   OMEGAX   OMEGAY   OMEGAZ   XORIG   YORIG   ZORIG\n')
            fstr = '{:>8d}{:>7d}{:>8d}{:>9.6f}{:>9.6f}{:>9.6f}'+'{:>8.5f}'*3+'\n'
            for blockId,vData in rotatedBlocks:
                xo,yo,zo = vData.get('axis',(0,0,1))
                rpm = vData['rpm']
                omega = rpm/60/self.env_acusitic_velocity
                norm0 = (xo**2+yo**2+zo**2)**0.5
                omegax = omega*xo/norm0
                omegay = omega*yo/norm0
                omegaz = omega*zo/norm0
                rfreq = 0
                self.fp.write(fstr.format(blockId,IROTAT,rfreq,omegax,omegay,omegaz,xo,yo,zo))
            if rotated_center:
                self.fp.write(fstr.format(0,IROTAT,rfreq,omegax,omegay,omegaz,xo,yo,zo))

            self.fp.write('    GRID   THXMAX   THYMAX   THZMAX\n')
            fstr = '{:>8d}'+' {:>9.6f}'*3+'\n'
            for blockId,vData in rotatedBlocks:
                xo,yo,zo = vData.get('axis',(0,0,1))
                deltaAngle = vData['max']
                norm0 = (xo**2+yo**2+zo**2)**0.5
                deltaAnglex = deltaAngle*xo/norm0
                deltaAngley = deltaAngle*yo/norm0
                deltaAnglez = deltaAngle*zo/norm0
                self.fp.write(fstr.format(blockId,deltaAnglex,deltaAngley,deltaAnglez))
            if rotated_center:
                self.fp.write(fstr.format(0,deltaAnglex,deltaAngley,deltaAnglez))

        else:
            tt = '    LREF\n    GRID IROTAT   RFREQ   OMEGAX   OMEGAY   OMEGAZ   XORIG   YORIG   ZORIG\n    GRID   THXMAX   THYMAX   THZMAX\n'
            self.fp.write(tt)

            
        
    def write_DYNAMIC_PATCH_block(self):
        patchedData = self.kargs.get('patchedData',None)
        if not (patchedData and self.kargs.get('IUNST')>0):
            return 
        ITOSS = self.kargs.get('ITOSS',0)
        ITMAX = self.kargs.get('ITMAX',0)
        if not isinstance(patchedData,str):
            patchedData = o2o.translatePatcheDataToString(self.blocks, patchedData, ITOSS, ITMAX)
        self.fp.write(patchedData.rstrip())
        self.fp.write('\n')

if __name__=='__main__':
    boundary={
        '1ke':{'bctype':2003},
        '4ke':{'bctype':2003},
        '5k0':{'bctype':2003},

        '1je':{'bctype':2004},
        '2je':{'bctype':2004},
        '3je':{'bctype':2004},
        '4je':{'bctype':2004},
        '5je':{'bctype':2004},

        '1j0':{'bctype':2004},
        '2j0':{'bctype':2004},
        '3j0':{'bctype':2004},
        '4j0':{'bctype':2004},
        '5j0':{'bctype':2004},

        '3i0':{'bctype':2004},

        '6je':{'bctype':-2004},
        '7je':{'bctype':-2004},
        '8je':{'bctype':-2004},
        '9je':{'bctype':-2004},
        '10je':{'bctype':-2004},

        '6j0':{'bctype':2004},
        '7j0':{'bctype':2004},
        '8j0':{'bctype':2004},
        '9j0':{'bctype':2004},
        '10j0':{'bctype':2004},

        '8i0':{'bctype':2004},

        '12k0':{'bctype':2002,'pressure':None},
        '14k0':{'bctype':2002,'pressure':None},
        '15ke':{'bctype':2002,'pressure':None},

        '15je':{'bctype':2004},
        '11je':{'bctype':2004},
        '12je':{'bctype':2004},
        '13je':{'bctype':2004},
        '14je':{'bctype':2004},

        '15j0':{'bctype':2004},
        '11j0':{'bctype':2004},
        '12j0':{'bctype':2004},
        '13j0':{'bctype':2004},
        '14j0':{'bctype':2004},

        '13i0':{'bctype':2004},

    }
    t = plot3d.read( '/home/yxs/F/pythonAPP/cfl3d_pyui/test_work/mesh_no_gap.g')

    k = o2o.one2one(t,periodic_faces=[((8,'IE'),(10,'IE'),True),((3,'IE'),(4,'IE'),True),((14,'IE'),(15,'IE'),True)])
    
    p1 = o2o.PatchedInterfaceRotated(t,[(14,'Ie'),], [(15,'Ie'),],0,0,True)
    p2 = o2o.PatchedInterfaceRotated(t,[(1,'K0'),(3,'K0'),(4,'Ke')], [(5,'Ke'),(8,'Ke'),(9,'Ie')],0,1)
    p3 = o2o.PatchedInterfaceRotated(t,[(11,'Ke'),(14,'Ke'),(15,'K0')], [(6,'K0'),(8,'K0'),(10,'Ke')],0,1)
    ttt = 2*math.pi
    print(p2[0][1]/3.1415926*180,'degree',ttt/p2[0][1])
    print(p2[1][1]/3.1415926*180,'degree',ttt/p2[1][1])
    print(p3[0][1]/3.1415926*180,'degree',ttt/p3[0][1])

    patchedData = o2o.patched([[o2o.BlockIdPlus(t) for t in p] for p in [p1,p2,p3]])

    blocks, oneData, patchedData = o2o.MergeBlocks(t,9,10,k,patchedData)

    rotatedBlocks={
        6:{'axis':(0,0,1),'max':14.4, 'rpm':10.},
        7:{'axis':(0,0,1),'max':14.4, 'rpm':10.},
        8:{'axis':(0,0,1),'max':14.4, 'rpm':10.},
        9:{'axis':(0,0,1),'max':14.4, 'rpm':10.},
        10:{'axis':(0,0,1),'max':14.4,'rpm':10.},
        11:{'axis':(0,0,1),'max':14.4,'rpm':10.},
    }
    #存在的问题，转子上壁面应该是一个绝对静止的无滑移边条，但是现在给的是一个bc2004边条
    #如果速度还有问题
    #在numeca上计算一个动网格算列
    a = inputFile(
        workdir='/home/yxs/F/pythonAPP/cfl3d_pyui/rotate_work',
        gridfile = blocks,
        input_filename='rotate5.inp',
        iterations=90,
        LREF = 1.0/1000,
        CFL_number=2,
        in_x_velocity = 0,in_y_velocity=0,in_z_velocity=200,
        in_density = 1.2,in_viscosity=1.716e-5,
        turbulence_model=5,
        output_pressure = 106000,
        I2D = 0,IUNST=1,MOVIE=100,rotated_center=True,NWREST=100,
        number_of_time_step=10000,time_step = 0.00005,
        BOUNDARY = boundary,
        oneOne = oneData,
        patchedData = patchedData,ITOSS = 0,ITMAX = 500,
        rotatedBlocks = rotatedBlocks,
        # restart_file = '/home/yxs/F/pythonAPP/cfl3d_pyui/rotate_work/rotate_restart.bin',
        ITA = -2,
        NCG = 0,
        init_file='/home/yxs/F/learning/CFL3D/323_0409/numeca_0409_323_steady/new_grid_323_0409_nogap/323_0409_steady/323_0409_steady/323_0409_steady_323_0411_STEADY/323_0409_steady_323_0411_STEADY.q',

    )
    a.start_write()



import postprocessing 
from yxspkg import tecfile 

if __name__=='__main__':
    cfl3d_case = '/home/yxs/F/learning/CFL3D/323_0409/ok/series/323_new_bound_0001/rotate323_0409_lref1_output.szplt'
    numeca_case='/home/yxs/F/learning/CFL3D/323_0409/ok/new_grid_323_0409_nogap/323_0409_steady/323_0409_steady/323_0409_steady_323_0411_STEADY_Frozen_106/323_0409_steady_323_0411_STEADY_Frozen_106.szplt'
    f = tecfile.read(numeca_case)
    postprocessing.cal_average(f)


import numpy as np
import math
#blocks is a list,the info of face include block id, the id start from 0 not 1
def to_fortran_format(func):
    def deal_r(r):
        a = [(i[0]+1,i[1]+1) for i in r[:-2]]
        a.extend((r[-2]+1,r[-1]+1))
        return a
    def wrap_func(*d,**k):
        a = func(*d,**k)
        if a is not None:
            a = [deal_r(i)  for i in a]
        return a
    return wrap_func
def is_equal_face(face1,face2,res = 1e-3):
    x1,y1,z1 = face1
    x2,y2,z2 = face2
    if x1.shape != x2.shape:
        return False
    if (np.abs(x1-x2).max() < res).all() and (np.abs(y1-y2).max() < res).all() and (np.abs(z1-z2).max() < res).all():
        return True 
    else:
        return False

def is_parallel_face(face1,face2,res = 1e-3):
    x1,y1,z1 = face1
    x2,y2,z2 = face2
    dx = np.abs(x1-x2)
    dy = np.abs(y1-y2)
    dz = np.abs(z1-z2)
    if (dx.max() - dx.min() < 2*res).all() and (dy.max() - dy.min()< 2*res).all() and (dz.max()-dz.min() < 2*res).all():
        return True 
    else:
        return False

def is_rotated_face(face1,face2,res = 1e-3):
    x1,y1,_ = face1
    x2,y2,_ = face2
    dx  = (x1*x2+y1*y2)/(np.sqrt(x1*x1+y1*y1)*np.sqrt(x2*x2+y2*y2))
    if (dx.max() - dx.min() < 0.02*res).all():
        return True 
    else:
        return False

def is_equal_point(p1,p2,res = 1e-3):
    x1, y1, z1 = p1 
    x2, y2, z2 = p2
    m = ((x2-x1)**2 + (y1-y2)**2 + (z1-z2)**2)**0.5
    if m < res:
        return True 
    else:
        return False
@to_fortran_format
def Ogrid_check(block,res=1e-3):
    x,y,z = block['X'],block['Y'],block['Z']
    idim,jdim,kdim = x.shape
    if abs(x[0,0,0] - x[-1,0,0]) < res:
        if np.abs(x[0]-x[-1]).max() < res and np.abs(y[0]-y[-1]).max() < res and np.abs(z[0]-z[-1]).max() < res:
            return ((0,0),(0,jdim-1),(0,kdim-1),1,2),((idim-1,idim-1),(0,jdim-1),(0,kdim-1),1,2) 
    if abs(y[0,0,0] - y[0,-1,0]) < res:
        if np.abs(x[:,0]-x[:,-1]).max() < res and np.abs(y[:,0]-y[:,-1]).max() < res and np.abs(z[:,0]-z[:,-1]).max() < res:
            return ((0,idim-1),(0,0),(0,kdim-1),0,2),((0,idim-1),(jdim-1,jdim-1),(0,kdim-1),0,2) 
    if abs(z[0,0,0] - z[0,0,-1]) < res:
        if np.abs(x[:,:,0]-x[:,:,-1]).max() < res and np.abs(y[:,:,0]-y[:,:,-1]).max() < res and np.abs(z[:,:,0]-z[:,:,-1]).max() < res:
            return ((0,idim-1),(0,jdim-1),(0,0),0,1),((0,idim-1),(0,jdim-1),(kdim-1,kdim-1),0,1) 
    return None
def point_in_face(point,face,return_position = False,res = 1e-3):
    #Is the point in the face, if True, return True, else return None
    x,y,z = face 
    px,py,pz = point
    tx = np.abs(x - px)
    ty = np.abs(y - py)
    tz = np.abs(z - pz)
    tx_min = tx.min()
    ty_min = ty.min()
    tz_min = tz.min()
    if tx_min < res and ty_min < res and tz_min < res:
        # there is a point of face2 belongs to face1
        if return_position:
            t = tx+ty+tz
            m = np.where(t == t.min())
            return m
        return True
    else:
        return False
# def cut_face(face,px1,py1,px3,py3,xstep,ystep):
#     if py3+ystep ==-1 and px3+xstep == -1:
#         face_temp = [i[px1::xstep,py1::ystep] for i in face]
#     elif px3 + xstep == -1:
#         face_temp = [i[px1::xstep,py1:py3+ystep:ystep] for i in face]
#     elif py3+ ystep == -1:
#         face_temp = [i[px1:px3+xstep:xstep,py1::ystep] for i in face]
#     else:
#         face_temp = [i[px1:px3+xstep:xstep,py1:py3+ystep:ystep] for i in face]
#     return face_temp
def get_double_face_x(face1, face2, point1, point3, xstep, ystep,message=None):
    x1,y1,z1 = face1 
    x2,y2,z2 = face2
    # print('shape',x2.shape)
    px1,py1 = point1 
    px3,py3 = point3
    (idim1,jdim1),(idim2,jdim2) = x1.shape,x2.shape
    result = []
    xstep_t = -xstep
    if py3+ystep == -1:
        face_temp1 = [x1[:px1+xstep_t:xstep_t,py1::ystep],
                    y1[:px1+xstep_t:xstep_t,py1::ystep],
                    z1[:px1+xstep_t:xstep_t,py1::ystep]]
    else:
        face_temp1 = [x1[:px1+xstep_t:xstep_t,py1:py3+ystep:ystep],
                    y1[:px1+xstep_t:xstep_t,py1:py3+ystep:ystep],
                    z1[:px1+xstep_t:xstep_t,py1:py3+ystep:ystep]]
    width1 = face_temp1[0].shape[0]
    face_temp2 = [x2[width1-1::-1],y2[width1-1::-1],z2[width1-1::-1]]
    if is_equal_face(face_temp1,face_temp2):
        if xstep<0:
            result.append(((px1,0),(py1,py3),0,1))
        else:
            result.append(((px1,idim1-1),(py1,py3),0,1))
        result.append(((0,width1-1),(0,jdim2-1),0,1))
    else:
        print('warning1 !!!!',message)
    #find the other
    # xstep_t = xstep
    if py3 + ystep == -1:
        face_temp1 = [x1[:px3+xstep:xstep,py1::ystep],
                    y1[:px3+xstep:xstep,py1::ystep],
                    z1[:px3+xstep:xstep,py1::ystep]]
     
    else:
        face_temp1 = [x1[:px3+xstep:xstep,py1:py3+ystep:ystep],
                    y1[:px3+xstep:xstep,py1:py3+ystep:ystep],
                    z1[:px3+xstep:xstep,py1:py3+ystep:ystep]]
    width2 = face_temp1[0].shape[0]

    face_temp2 = [x2[idim2-width2:],
                    y2[idim2-width2:],
                    z2[idim2-width2:]]
    # print(face_temp1[0].shape,face_temp2[0].shape)
    if is_equal_face(face_temp1,face_temp2):
        if xstep>0:
            result.append(((0,px3),(py1,py3),0,1))
        else:
            result.append(((idim1-1,px3),(py1,py3),0,1))
        result.append(((idim2-width2,idim2-1),(0,jdim2-1),0,1))
    else:
        print('warning2 !!!!!!!!!',message)
        print('ttt',result)
    if len(result) == 0:
        result = not_connected(45,message)
    return result
def get_matched_face(face1,face2,point1,point3,xstep,ystep,message):
    px1,py1 = point1 
    px3,py3 = point3
    if (px1 - px3) * xstep>0:
        #across x axis
        return get_double_face_x(face1,face2,point1,point3,xstep,ystep,message)
    elif (py1 - py3) * ystep>0:
        #across y axis
        face1_T = [i.T for i in face1]
        face2_T = [i.T for i in face2]
        point1_T = py1,px1 
        point3_T = py3,px3
        result = get_double_face_x(face1_T,face2_T,point1_T, point3_T, ystep, xstep,message)
        if result:
            result = [(i2,i1,i3,i4) for i1,i2,i3,i4 in result]
        return result
    else:
        # x_index = range(px1, px3+xstep, xstep)
        # y_index = range(py1, py3+ystep, ystep)
        # print(x_index,y_index,x1.shape)
        if py3+ystep ==-1 and px3+xstep == -1:
            face1_temp = [i[px1::xstep,py1::ystep] for i in face1]
        elif px3 + xstep == -1:
            face1_temp = [i[px1::xstep,py1:py3+ystep:ystep] for i in face1]
        elif py3+ ystep == -1:
            face1_temp = [i[px1:px3+xstep:xstep,py1::ystep] for i in face1]
        else:
            face1_temp = [i[px1:px3+xstep:xstep,py1:py3+ystep:ystep] for i in face1]
        if is_equal_face(face1_temp,face2):
            idim2,jdim2 = face2[0].shape
            result = ((px1,px3),(py1,py3),0,1),((0,idim2-1),(0,jdim2-1),0,1)
            return result
    return None

def not_connected(code=0,message='no message', warningOutput=False):
    #没有匹配到网格，但是这个情况有点奇怪
    if warningOutput:
        print("warning!please insure your grid is correct! code:",code,message)
    return None
def SmallInBig(face1,face2,face1_info,face2_info,res = 1e-3,message = None, warningOutput = True):
    #there is one point of face2 belong to face1
    oblock1,oblock2 = message[-2:]
    x1,y1,z1 = face1
    x2,y2,z2 = face2
    idim1,jdim1 = x1.shape
    idim2,jdim2 = x2.shape
    if idim1*jdim1 < idim2*jdim2:
        exchange = True
        face1,face2 = face2,face1
        face1_info,face2_info = face2_info,face1_info
        x1,y1,z1 = face1
        x2,y2,z2 = face2
        idim1,jdim1 = x1.shape
        idim2,jdim2 = x2.shape
        oblock1,oblock2 = oblock2, oblock1
    else:
        exchange = False
    p3 = x2[-1,-1],y2[-1,-1],z2[-1,-1]
    
    pif3 = point_in_face(p3,face1,return_position=True)

    if pif3:
        p1 = x2[0,0],y2[0,0],z2[0,0]
        pif = point_in_face(p1,face1,return_position=True)
        # print('pif3',x2.shape)
        if pif:
            # print('pif1')
            px1,py1 = pif
            px3,py3 = pif3
            
            px1,py1 = px1[0],py1[0]
            px3,py3 = px3[0],py3[0]


            p2 = x2[0,-1],y2[0,-1],z2[0,-1]
            pif2 = point_in_face(p2,face1,return_position=True)
            if pif2:
                px2, py2 = pif2 
                px2, py2 = px2[0],py2[0]
                # 其中face2的三个顶点都在face1上，再检查face2面上的非顶点的一个点是否在face1上
                # 避免两个面组成环的形式出现
            
                p_in = x2[1,1],y2[1,1],z2[1,1]
                pif_in = point_in_face(p_in,face1,return_position=False)
                if not pif_in:
                    return not_connected('circle',message, warningOutput)
            else:
                return not_connected(2,message, warningOutput)
            if px1 == px2 == px3 or py1 == py2 == py3:
                #三个点在一条线
                return None
            
            #检查是否需要转置后再对应

            if px2 == px1:
                is_transpose = False 
            else:
                is_transpose = True
            
            if is_transpose:
                x2, y2, z2 = x2.T, y2.T, z2.T
                face2 = [x2, y2, z2]
                idim2, jdim2 = x2.shape

            #检查轴的对应方向
            if px2>px3:
                xstep = -1 
            else:
                xstep = 1 
            if py1>py2:
                ystep = -1
            else:
                ystep = 1
            pdy = x2[0,1], y2[0,1], z2[0,1]
            pdx = x2[1,0], y2[1,0], z2[1,0]

            if idim1 > px1+xstep and jdim1 > py1:
                f1pdx = x1[px1+xstep,py1],y1[px1+xstep,py1],z1[px1+xstep,py1]
            else:
                if idim1 == px1+xstep and xstep == 1:
                    f1pdx = x1[0,py1],y1[0,py1],z1[0,py1]
                else:
                    raise Exception('condition not find of x')
            if idim1 > px1 and jdim1 > py1+ystep: 
                f1pdy = x1[px1,py1+ystep],y1[px1,py1+ystep],z1[px1,py1+ystep]
            else:
                if jdim1 == py1+ystep and ystep == 1:
                    f1pdy = x1[px1,0],y1[px1,0],z1[px1,0]
                else:
                    raise Exception('condition not find of y')

            if not is_equal_point(pdx,f1pdx):
                xstep = -xstep
                #往相反的方向检查 也可能因为网格数对不上 而出错 而这种情况可能是不存在正确匹配面的，所以排除
                if xstep+px1 >= x1.shape[0]:
                    return None

                f1pdx2 = x1[px1+xstep,py1],y1[px1+xstep,py1],z1[px1+xstep,py1]
                if not is_equal_point(pdx,f1pdx2):
                    if oblock1 is True:
                        return None 
                    else:
                        return not_connected(12,message)
            if not is_equal_point(pdy,f1pdy):
                ystep = -ystep
                #往相反的方向检查 也可能因为网格数对不上 而出错 而这种情况可能是不存在正确匹配面的，所以排除
                if py1+ystep >= x1.shape[1]:
                    return
                f1pdy2 = x1[px1,py1+ystep],y1[px1,py1+ystep],z1[px1,py1+ystep]
                if not is_equal_point(pdy,f1pdy2):
                    if oblock1 is True:
                        return None 
                    else:
                        return not_connected(344 ,message,warningOutput)

            point1 = px1,py1 
            point3 = px3,py3
            result = get_matched_face(
                face1, face2, point1, point3, xstep, ystep,message
            )
            if result:
                if len(result) == 2:
                    r1, r2 = result
                    if is_transpose:
                        r2 = r2[1],r2[0],r2[3],r2[2]
                    if exchange:
                        result = r2,r1 
                    else:
                        result = r1,r2 
                    return result
                elif len(result) == 4:
                    r1,r2,r3,r4 = result
                    if is_transpose:
                        r2 = r2[1],r2[0],r2[3],r2[2]
                        r4 = r4[1],r4[0],r4[3],r4[2]
                    if exchange:
                        result = r2,r1,r4,r3
                    else:
                        result = r1,r2,r3,r4
                    return result
            else:
                if oblock1 is True and oblock2 is True:
                    return None 
                else:
                    return not_connected(4,message, warningOutput)
    return not_connected('没有找到',message, warningOutput)
def FindPointInLines(points, lines, res = 1e-3):
    length = points.shape[0]
    p = points[0]
    for k,line in enumerate(lines):
        t1 = np.abs(line - p ).sum(1).min()
        if t1 < res:
            return 0,k
    for i in range(6,-1,-1):
        step = 2**i
        for j in range(0,length,step):
            if j % (step*2) == 0:
                continue 
            p = points[j]
            for k,line in enumerate(lines):
                t1 = np.abs(line - p ).sum(1).min()
                if t1 < res:
                    return j,k
    return None, None
def FindBoundPointInLine(pStart,pEnd,points,line,res = 1e-5):
    # res = 
    distance = lambda line0,p0:1 if np.abs(line0 - p0).sum(1).min() < res else -1
    stateS = distance(line,points[pStart])
    # step = 1 if pEnd > pStart else -1
    # for ii in range(pStart,pEnd+step,step):
    #     if distance(line,points[ii]) == 1:
    #         print(ii)
    #         break
    while True:
        pM = (pEnd+pStart) // 2
        stateM = distance(line,points[pM])
        if stateM*stateS<0:
            pEnd = pM 
            # stateE = stateM
        else:
            pStart = pM 
            stateS = stateM
        if pEnd - pStart <= 1:
            if stateS == 1:
                # print('start',pStart,pEnd)
                return pStart 
            else:
                # print('end',pStart,pEnd)
                return pEnd
def FindPartInFace(face1,face2,res = 1e-3):
    x1,y1,z1 = face1
    x2,y2,z2 = face2
    points0 = np.vstack([x2[:,0], y2[:,0], z2[:,0]]).T
    points1 = np.vstack([x2[:,-1], y2[:,-1], z2[:,-1]]).T
    line1 = np.vstack([x1[:,0],y1[:,0],z1[:,0]]).T
    line2 = np.vstack([x1[:,-1],y1[:,-1],z1[:,-1]]).T
    lines = [line1, line2]
    pN,lineN = FindPointInLines(points0,lines)
    if pN is None:
        return None,None
    
    p1 = FindBoundPointInLine(0,pN,points0,lines[lineN])
    # print(p1,pN,"7777777777")
    p2 = FindBoundPointInLine(pN,points0.shape[0],points0,lines[lineN])
    if np.abs(lines[1-lineN] - points1[p1]).sum(1).min()>res:
        return None,None
    if np.abs(lines[1-lineN] - points1[p2]).sum(1).min()>res:
        return None,None

    if p1 == p2:
        return None,None
    else:
        return p1,p2
@to_fortran_format
def face2face(face1,face2,face1_info,face2_info,res = 1e-3,message = None):
    def TransPartConnectResult(r1,r2,exchange,face1_T,face2_T,p1):

        r2 = (r2[0][0]+p1,r2[0][1]+p1),r2[1],r2[2],r2[3]
        if exchange:
            r1,r2 = r2,r1 
            face1_T,face2_T = face2_T,face1_T
        if face1_T:
            r1 = r1[1], r1[0], r1[3], r1[2]
        if face2_T:
            r2 = r2[1], r2[0], r2[3], r2[2]
        return r1,r2

    oblock1,oblock2 = message[-2:]
    (x1min,x1max),(y1min,y1max),(z1min,z1max) = face1_info
    (x2min,x2max),(y2min,y2max),(z2min,z2max) = face2_info
    if x2min - x1max > res or x1min - x2max > res:

        return None
    if y2min - y1max > res or y1min - y2max > res:

        return None
    if z2min - z1max > res or z1min - z2max > res:
 
        return None
    x1,y1,z1 = face1
    x2,y2,z2 = face2
    idim1,jdim1 = x1.shape
    idim2,jdim2 = x2.shape
    if x1.shape == x2.shape:
        if np.abs(x1 - x2).max() < res and np.abs(y1 - y2).max() < res and np.abs(z1 - z2).max() < res:
            return ((0,idim1-1),(0,jdim1-1),0,1),((0,idim2-1),(0,jdim2-1),0,1)
    if (idim1>=idim2 and jdim1>=jdim2) or (idim1<=idim2 and jdim1<=jdim2) or \
        (idim1>=jdim2 and jdim1>=idim2) or (idim1<=jdim2 and jdim1<=idim2):
        #face1 is a big face, face2 is included in face1,if face1 is smaller than face2, exchange them.
        # print('first ')
        a = SmallInBig(face1,face2,face1_info,face2_info,message=message, warningOutput=False)
        # a = None
        if a:
            return a
        else:
            #满足一个面比一个面小，但是小的面上的顶点不在，这时候face1是大的面face2是小的面  注意后面进行交换
            #确保第face1是大面
            if idim1*jdim1 < idim2*jdim2:
                exchange = True
                face1,face2 = face2,face1
                face1_info,face2_info = face2_info,face1_info
                x1,y1,z1 = face1
                x2,y2,z2 = face2
                idim1,jdim1 = x1.shape
                idim2,jdim2 = x2.shape
                message = list(message)
                message[-2],message[-1] = oblock2, oblock1
            else:
                exchange = False

            #确保face1的j轴小于等于i轴
            if jdim1 > idim1:
                face1_T = True 
                x1,y1,z1 = face1
                face1 = x1.T, y1.T, z1.T 
                x1,y1,z1 = face1
                idim1,jdim1 = jdim1, idim1
            else:
                face1_T = False

            if jdim1 == jdim2 or jdim1 == idim2:
                p1 = None
                if idim2 == jdim2:
                    p1,p2 = FindPartInFace(face1,face2)
                    if p1 is not None:
                        equalCheck = True 
                    else:
                        equalCheck = False
                else:
                    equalCheck = False
                if jdim1 == idim2 and equalCheck is False:
                    face2_T = True 
                    x2,y2,z2 = face2
                    face2 = x2.T, y2.T, z2.T 
                    x2,y2,z2 = face2
                    idim2,jdim2 = jdim2, idim2
                else:
                    face2_T = False
                if p1 is None:
                    p1,p2 = FindPartInFace(face1,face2)
                    # print(p1,p2)
         
                if p1 is not None:
                    # print(p1,p2)
                    # p1+=1
                    face2 = x2[p1:p2+1], y2[p1:p2+1], z2[p1:p2+1]
                    # print(face2[1].max(), face2[1].min())
                    # print('second')
                    t = SmallInBig(face1,face2,face1_info,face2_info,message=message)
                    if t:
                        if len(t) == 2:
                            r1,r2 = t
                            return TransPartConnectResult(r1,r2,exchange,face1_T,face2_T,p1)
                        elif len(t) is 4:
                            r1,r2,r3,r4 = t
                            r1,r2 = TransPartConnectResult(r1,r2,exchange,face1_T,face2_T,p1)
                            r3,r4 = TransPartConnectResult(r3,r4,exchange,face1_T,face2_T,p1)
                            return r1,r2,r3,r4

            else:
                return not_connected("四个顶点都不在面上 且没有一条边是相等的",message)
    else:
        return not_connected('不存在所属关系',message)

    return None

def generate_blocks_info(blocks):
    def get_block_info(block):
        face_info = dict()
        x,y,z = block['X'],block['Y'],block['Z']
        face_info['K0'] = (x[:,:,0].min(), x[:,:,0].max()), ( y[:,:,0].min(), y[:,:,0].max()),  (z[:,:,0].min(), z[:,:,0].max())
        face_info['KE'] = (x[:,:,-1].min(),x[:,:,-1].max()), (y[:,:,-1].min(),y[:,:,-1].max()), (z[:,:,-1].min(),z[:,:,-1].max())
        face_info['J0'] = (x[:,0,:].min(), x[:,0,:].max()), ( y[:,0,:].min(), y[:,0,:].max()),  (z[:,0,:].min(), z[:,0,:].max())
        face_info['JE'] = (x[:,-1,:].min(),x[:,-1,:].max()), (y[:,-1,:].min(),y[:,-1,:].max()), (z[:,-1,:].min(),z[:,-1,:].max())
        face_info['I0'] = (x[0,:,:].min(), x[0,:,:].max()), ( y[0,:,:].min(), y[0,:,:].max()),  (z[0,:,:].min(), z[0,:,:].max())
        face_info['IE'] = (x[-1,:,:].min(),x[-1,:,:].max()), (y[-1,:,:].min(),y[-1,:,:].max()), (z[-1,:,:].min(),z[-1,:,:].max())
        return face_info
    return [get_block_info(i) for i in blocks]
def generate_blocks_face(blocks):
    def get_block_faces(block):
        faces = dict()
        x,y,z = block['X'],block['Y'],block['Z']
        faces['K0'] = x[:,:,0],  y[:,:,0],  z[:,:,0]
        faces['KE'] = x[:,:,-1], y[:,:,-1], z[:,:,-1]
        faces['J0'] = x[:,0,:],  y[:,0,:],  z[:,0,:]
        faces['JE'] = x[:,-1,:], y[:,-1,:], z[:,-1,:]
        faces['I0'] = x[0,:,:],  y[0,:,:],  z[0,:,:]
        faces['IE'] = x[-1,:,:], y[-1,:,:], z[-1,:,:]
        return faces 
    return [get_block_faces(block) for block in blocks]

def one2one(blocks,periodic_faces=None,periodic_faces_result = None,periodic_only = False):
    #perodic_faces is list like  [((block1,face1),(block2,face2),rotated),....]
    #example [((0,'I0'),(1,'IE'),False),...],block id is ctype, start from 0
    def get_periodic_():
        for (id1,key1),(id2,key2),rotated in periodic_faces:

            face1 = blocks_faces[id1][key1]
            face2 = blocks_faces[id2][key2]
            # if not rotated:
            rr = get_periodic_faces(face1,face2,rotated)
        
            if rr is not None:
                (r1,r2) = rr
                r1 = expand_to_3d(key1,r1,blocks[id1]['X'].shape)
                r2 = expand_to_3d(key2,r2,blocks[id2]['X'].shape)
                result.append((id1+1,r1,id2+1,r2))
            # else:
            #     r1,r2 = get_periodic_faces(face1,face2,rotated)
            #     raise Exception('rotated periodic faces')

    

    def expand_to_3d(key,result,shape):
        #因为face2face返回的结果是针对一个二维的平面，并不包含第三个轴，所以需要转换到和Ogrid_check的结果一样的结果
        idim,jdim,kdim = shape 
        r1,r2,index1,index2 = result
        if key[0] == 'I':
            if key[1] == '0':
                result0 = (1,1), r1, r2, index1+1,index2+1
            else:
                result0 = (idim,idim), r1, r2, index1+1,index2+1
        elif key[0] == 'J':
            if key[1] == '0':
                result0 = r1, (1,1), r2, index1,index2+1
            else:
                result0 = r1, (jdim,jdim), r2, index1,index2+1
        elif key[0]=='K':
            if key[1] == '0':
                result0 = r1,r2, (1,1),index1,index2
            else:
                result0 = r1,r2, (kdim,kdim),index1,index2
        else:
            raise Exception('key is an error variables x ,y,z not in key.')
        return result0

    blocks_info = generate_blocks_info(blocks)
    blocks_faces = generate_blocks_face(blocks)
    result = []
    keys = ['K0','KE','J0','JE','I0','IE']
    try:
        oblocks = dict()
        for i,block in enumerate(blocks):
            k = Ogrid_check(block)
            if k is not None:
                oblocks[i] = True 
                result.append((i+1,k[0],i+1,k[1]))
        if periodic_faces:
            length1 = len(result)
            get_periodic_()
            if periodic_faces_result is not None:
                periodic_faces_result.extend(result[length1:])
        if periodic_only:
            return result
        for i,block in enumerate(blocks):
            # if i != 3:
            #     continue
            for j in range(len(blocks)):
                # if j != 6:
                #     continue
                if i>=j:
                    continue
                for key1 in keys:
                    face1 = blocks_faces[i][key1]
                    info1 = blocks_info[i][key1]
                    for key2 in keys:
                        # if key1 != 'KE' or key2 != 'IE':
                        #     continue
                        face2 = blocks_faces[j][key2]
                        info2 = blocks_info[j][key2]
                        t = face2face(face1,face2,info1,info2,message=(i+1,j+1,key1,key2,oblocks.get(i,False),oblocks.get(j,False)))

                        if t is not None:
                            if len(t) == 2:
                                r1,r2 = t
                             
                                r1 = expand_to_3d(key1,r1,block['X'].shape)
                                r2 = expand_to_3d(key2, r2, blocks[j]['X'].shape)
                                result.append((i+1,r1,j+1,r2))
                            elif len(t) == 4:
                                r1,r2,r3,r4 = t 
                                r1 = expand_to_3d(key1,r1,block['X'].shape)
                                r2 = expand_to_3d(key2, r2, blocks[j]['X'].shape)
                                r3 = expand_to_3d(key1,r3,block['X'].shape)
                                r4 = expand_to_3d(key2, r4, blocks[j]['X'].shape)
                                result.extend(((i+1,r1,j+1,r2),(i+1,r3,j+1,r4)))

    except Exception as e:
        print(e)
        print(i,j,key1,key2,'yyyyyyyyyyyyyyyyyyyyy')
        raise Exception('stop')
    #检查是否通过oneone检测
    Varify_one2one(blocks,result)

    return result
def Varify_one2one(blocks,result):
    def get_face(block,r,bid):
        ((i1,i2),(j1,j2),(k1,k2),ind1,ind2) = r
        x,y,z = block['X'],block['Y'],block['Z']
        istep = 1 if i2>i1 else -1
        jstep = 1 if j2>j1 else -1
        kstep = 1 if k2>k1 else -1
        i1,j1,k1 = i1-1,j1-1,k1-1
        inds = ind1 + ind2
        if 6 - inds == 1:
            h,w = abs(j2-j1-1)+1,abs(k2-k1-1)+1
            face = x[i1,j1::jstep,k1::kstep][:h,:w],y[i1,j1::jstep,k1::kstep][:h,:w],z[i1,j1::jstep,k1::kstep][:h,:w]
        elif 6 - inds == 2:
            h,w = abs(i2-i1-1)+1,abs(k2-k1-1)+1
            face = x[i1::istep,j1,k1::kstep][:h,:w],y[i1::istep,j1,k1::kstep][:h,:w],z[i1::istep,j1,k1::kstep][:h,:w]
        elif 6 - inds == 3:
            h,w = abs(i2-i1-1)+1,abs(j2-j1-1)+1
            face = x[i1::istep,j1::jstep,k1][:h,:w],y[i1::istep,j1::jstep,k1][:h,:w],z[i1::istep,j1::jstep,k1][:h,:w]
        else:
            raise Exception('One2One check error')
        if ind1 > ind2:
            face = face[0].T,face[1].T,face[2].T 
        return face
    for bid1,r1,bid2,r2 in result:
        f1 = get_face(blocks[bid1-1],r1,bid1)
        f2 = get_face(blocks[bid2-1],r2,bid2)
        t = is_equal_face(f1,f2)
        if t is False:
            t = is_parallel_face(f1,f2)
        if t is False:
            t = is_rotated_face(f1,f2)
        if t is False:
            print(bid1,r1,bid2,r2)
            raise Exception('One2One check error 12')
def transfer_one2one_str(result):
    result.sort(key=lambda x:x[0])
    def expand_var(vv):
        GRID,((ISTA,IEND),(JSTA,JEND),(KSTA,KEND),ISVA1,ISVA2) = vv
        return GRID,ISTA,JSTA,KSTA,IEND,JEND,KEND,ISVA1,ISVA2
    bstr = ['   1-1 BLOCKING DATA:']
    bstr.append('{:>10s}'.format('NBLI'))
    bstr.append('{:>10d}'.format(len(result)))
    title = '   NUMBER   GRID   :   ISTA   JSTA   KSTA   IEND   JEND   KEND   ISVA1   ISVA2'
    bstr.append(title)
    str_format = '{:>9d}{:>7d}{:>11d}'+'{:>7d}'*5+'{:>8d}'*2
    bstr += [str_format.format(i+1,*expand_var(v[:2])) for i,v in enumerate(result)]
    bstr.append(title)
    bstr += [str_format.format(i+1,*expand_var(v[2:])) for i,v in enumerate(result)]

    return '\n'.join(bstr)
def get_periodic_faces(face1,face2,rotated = False,res = 1e-3):
    #该函数返回结果是 fortran类型 start from 1

    if not rotated:
        x1,y1,z1 = face1 
        x2,y2,z2 = face2
        px1,py1,pz1 = 0,0,0
        px2,py2,pz2 = 0,0,0
        for i,j in [(0,0),(-1,-1),(0,-1),(-1,0)]:
            px1+=x1[i,j]
            py1+=y1[i,j]
            pz1+=z1[i,j]
            px2+=x2[i,j]
            py2+=y2[i,j]
            pz2+=z2[i,j]
        x2 =x2 - (px2-px1)/4
        y2 =y2 - (py2-py1)/4
        z2 =z2 - (pz2-pz1)/4
        face2 = x2,y2,z2
        a = SmallInBig(face1,face2,None,None,message=(False,False))
        if a:
            r1,r2 = a 
            r1 = ((r1[0][0]+1,r1[0][1]+1),(r1[1][0]+1,r1[1][1]+1),r1[2]+1,r1[3]+1)
            r2 = ((r2[0][0]+1,r2[0][1]+1),(r2[1][0]+1,r2[1][1]+1),r2[2]+1,r2[3]+1)
        rr = r1,r2
        return rr
    else:
        return get_rotated_periodic_faces(face1,face2)
def get_rotated_periodic_faces(face1,face2,res = 1e-3):
    x1,y1,z1 = face1
    x2,y2,z2 = face2
    idim1,jdim1 = x1.shape 
    idim2,jdim2 = x2.shape 
    
    if idim1 == idim2 and jdim1 == jdim2 :
        if (abs(z2[0,0] - z1[-1,0])<res and abs(z2[0,-1] - z1[-1,-1]) < res) or \
        (abs(z2[0,0] - z1[-1,-1])<res and abs(z2[0,-1] - z1[-1,0]) < res):
            iTranspose = -1
            x2, y2, z2 = x2[::-1],y2[::-1],z2[::-1]
        else:
            iTranspose = 1
        d1 = (x1[(0,-1),:][:,(0,-1)]**2 + y1[(0,-1),:][:,(0,-1)]**2)**0.5
        d2 = (x2[(0,-1),:][:,(0,-1)]**2 + y2[(0,-1),:][:,(0,-1)]**2)**0.5
        if (abs(d2[0,0] - d1[0,-1])<res and abs(d2[0,-1] - d1[0,0]) < res):
            jTranspose = -1
            x2, y2, z2 = x2[:,::-1],y2[:,::-1],z2[:,::-1]
        else:
            jTranspose = 1
        da = (x2*x1 + y2*y1)/(((x2**2+y2**2)*(x1**2+y1**2))**(1/2))
        
        if da.max() - da.min() > res:
            
            print('Warning these two faces may not be rotated periodic',da.max() - da.min())
            return None
        return ((1,idim1),(1,jdim1),1,2), ((1,idim2)[::iTranspose],(1,jdim2)[::jTranspose],1,2)
    else:
        
        print(idim1,idim2,jdim1,jdim2,'\nthis conditon may not be concerned!!')
        return None

        raise Exception('this conditon may not be concerned!!')

def GetFace(block,faceId):
    x,y,z = block['X'], block['Y'], block['Z']
    if faceId[1] == '0':
        dim = 0 
    else:
        dim = -1
    if faceId[0].upper() == 'I':
        return x[dim],y[dim],z[dim]
    elif faceId[0].upper() == 'J':
        return x[:,dim],y[:,dim],z[:,dim]
    elif faceId[0].upper() == 'K':
        return x[:,:,dim],y[:,:,dim],z[:,:,dim]
    else:
        raise Exception('faceId error',faceId)
def GetFaceAngle(point0,face,pos=None):
    #默认旋转轴是z轴
    def get_p_list(face,pos):
        x,y = face[:2]
        p_list = [(0,0),(0,-1),(-1,-1),(-1,0)]
        distance = [((i,j),math.sqrt(x[i,j]**2+y[i,j]**2)) for i,j in p_list]
        distance.sort(key = lambda k:k[1])
        # print(distance,'distahce')
        distance = [i[0] for i in distance]
        if pos.lower() == 'top':
            return distance[-2:]
        else:
            return distance[:2]
    px,py = point0[:2]
    x,y = face[:2]
    norm0 = math.sqrt(px*px+py*py)
    amax,amin = -600,600
    if pos is None:
        p_list = [(0,0),(0,-1),(-1,-1),(-1,0)]
    else:
        p_list = get_p_list(face,pos)
    for i, j in p_list:
        px1,py1 = x[i,j], y[i,j]
        fz = px*px1 + py*py1 
        fm = norm0*math.sqrt(px1*px1+py1*py1)
        t = fz/fm
        t = 1 if t>1 else t 
        t = -1 if t<-1 else t
        angle = math.acos(t)
        if px*py1 - py*px1<0:
            angle = -angle
        amin = min(amin,angle)
        amax = max(amax,angle)
    return amax - amin ,amin,amax
# def GetLinesAngle(point0,lines):
#     #默认旋转轴是z轴
#     px,py = point0[:2]
#     norm0 = math.sqrt(px**2 + py**2)
#     amax,amin = -600,600
#     for pxpy in lines:
#         for px1,py1 in pxpy:
#             fz = px*px1 + py*py1 
#             fm = norm0*math.sqrt(px1*px1+py1*py1)
#             t = fz/fm
#             t = 1 if t>1 else t 
#             t = -1 if t<-1 else t
#             angle = math.acos(t)
#             if px*py1 - py*px1<0:
#                 angle = -angle
#             amin = min(amin,angle)
#             amax = max(amax,angle)
#     return amax - amin ,amin,amax
def FindBottomTopLine(surf):
    in_face_x, in_face_y = surf
    point00 = in_face_x[0,0],in_face_y[0,0]
    point01 = in_face_x[0,-1],in_face_y[0,-1]
    point11 = in_face_x[-1,-1],in_face_y[-1,-1]
    point10 = in_face_x[-1,0],in_face_y[-1,0]
    
    point_list = [point00,point01,point11,point10]
    distance = []
    for i,(px,py) in enumerate(point_list):
        if i == 3:
            j = 0
        else:
            j = i+1
        p2x,p2y = point_list[j]
        pxm,pym = (px+p2x)/2 , (py+p2y)/2
        distance.append((i,j,math.sqrt(pxm**2+pym**2)))
    distance.sort(key = lambda x:x[-1])
    in_top_line = distance[-1]
    in_bottom_line = distance[0]
    return in_top_line,in_bottom_line,point_list

def GetFacesAngle(point0,faces,position = 'Top'):
    bottom_top_lines = [FindBottomTopLine(f[:2]) for f in faces]
    
    bottom_lines = [(points[ib],points[jb]) for (it,jt,_),(ib,jb,_),points in bottom_top_lines]
    if position == 'Top':
        top_lines = [(points[it],points[jt]) for (it,jt,_),(ib,jb,_),points in bottom_top_lines]
        return GetLinesAngle(point0,top_lines)
    else:
        bottom_lines = [(points[ib],points[jb]) for (it,jt,_),(ib,jb,_),points in bottom_top_lines]
        return GetLinesAngle(point0, bottom_lines)

# def PatchedInterfaceRotatedSingle(blocks,faces1,faces2,periodic1 = 0,periodic2 = 1,position = 'Top'):
#     #faces1:[(2,'I0'),(3,'J0')]
#     #默认face2是旋转面
#     fs1 = [GetFace(blocks[i], faceId) for i,faceId in faces1]
#     fs2 = [GetFace(blocks[i], faceId) for i,faceId in faces2]
    

#     fx,fy,_ = fs1[0]
#     point0 = fx[0,0], fy[0,0]
#     GetFacesAngle(point0,fs1,position)

#     angleSpan1, angle1Min, angle1Max = GetFacesAngle(point0,fs1,position)
#     angleSpan2, angle2Min, angle2Max = GetFacesAngle(point0,fs2,position)
    

#     # angle2Min = min([i[1] for i in angles2])
#     # angle2Max = max([i[2] for i in angles2])
#     # angleSpan2 = angle2Max - angle2Min

#     # angleSpan1 = 2*np.pi/round(2*np.pi/angleSpan1)
#     # angleSpan2 = 2*np.pi/round(2*np.pi/angleSpan2)
#     #旋转过后的角度
#     if periodic1>0:
#         angle1MaxRotated = angle1Max + angleSpan1*periodic1 
#         angle1MinRotated = angle1Min 
#     elif periodic1<0:
#         angle1MaxRotated = angle1Max 
#         angle1MinRotated = angle1Min + angleSpan1*periodic1 
#     else:
#         angle1MaxRotated = angle1Max 
#         angle1MinRotated = angle1Min
    
#     if periodic2>0:
#         angle2MaxRotated = angle2Max + angleSpan2*periodic2
#         angle2MinRotated = angle2Min 
#     elif periodic2<0:
#         angle2MaxRotated = angle2Max 
#         angle2MinRotated = angle2Min + angleSpan2*periodic2
#     else:
#         angle2MaxRotated = angle2Max 
#         angle2MinRotated = angle2Min 
#     # if not patchedPeriodic:
#     f1tof2Positive = math.ceil((angle2MaxRotated - angle1MinRotated)/angleSpan1)
#     f1tof2Negative = -(math.ceil((angle1MaxRotated - angle2MinRotated )/angleSpan1) - 1)

#     f2tof1Positive = math.ceil((angle1MaxRotated - angle2MinRotated)/angleSpan2)
#     f2tof1Negative = -(math.ceil((angle2MaxRotated - angle1MinRotated )/angleSpan2) - 1)

#     return (faces1, angleSpan1, f1tof2Negative, f1tof2Positive), (faces2, angleSpan2, f2tof1Negative, f2tof1Positive)
#     # else:
#     #     assert abs(angleSpan1 - angleSpan2)<1e-3
#     #     dt = (angle2Max - angle1Max)/angleSpan1
#     #     return (faces1, angleSpan1, dt, dt), (faces2, angleSpan2, -dt, -dt)
def PatchedInterfaceRotatedPeriodic(blocks,faces1,faces2):
    #计算旋转周期壁面，该壁面无法进行oneoneblock赋值，只能进行插值计算
    fs1 = [GetFace(blocks[i], faceId) for i,faceId in faces1]
    fs2 = [GetFace(blocks[i], faceId) for i,faceId in faces2]
    

    fx,fy,_ = fs1[0]
    point0 = fx[0,0], fy[0,0]

    angels1 = [GetFaceAngle(point0,i) for i in fs1]
    angels2 = [GetFaceAngle(point0,i) for i in fs2]
    ma1_list = [i[2] for i in angels1]
    ma2_list = [i[2] for i in angels2]
    ma1,ma2 = max(ma1_list),max(ma2_list)
    span = ma2 - ma1 
    return (faces1,span,1,1), (faces2, span, -1, -1)

# def PatchedInterfaceRotated(blocks,faces1,faces2,periodic1 = 0,periodic2 = 1,patchedPeriodic = False):
#     if patchedPeriodic:
#         return PatchedInterfaceRotatedPeriodic(blocks, faces1, faces2)
#     (f1,a1,n1,p1),(f2,a2,n2,p2) = PatchedInterfaceRotatedSingle(blocks,faces1,faces2,periodic1 ,periodic2  ,position = 'Top')
#     (_,a3,n11,p11),(_,a4,n22,p22) = PatchedInterfaceRotatedSingle(blocks,faces1,faces2,periodic1 ,periodic2 ,position = 'Bottom')
#     t = (f1,a1,min(n1,n11),max(p1,p11)),(f2,a2,min(n2,n22),max(p2,p22))
#     # print(a1,a2,n1,p1,n2,p2)
#     # print(a1,a3,n11,p11,n22,p22)
#     return t
def MergeBlocks(blocks,b1,b2,one2oneData=None,patchedData=None,boundaryData=None):
    #b1,b2 start from 0
    if b2 < b1:
        b1, b2 = b2, b1
    block1 = blocks[b1]
    block2 = blocks[b2]
    shape1, shape2 = block1['X'].shape, block2['X'].shape
    def detectEqualFace(dim,pos):
        if dim == 0:
            face1 = block1['X'][pos],block1['Y'][pos],block1['Z'][pos]
            face20 = block2['X'][0],block2['Y'][0],block2['Z'][0]
            face21 = block2['X'][-1],block2['Y'][-1],block2['Z'][-1]
        elif dim == 1:
            face1 = block1['X'][:,pos],block1['Y'][:,pos],block1['Z'][:,pos]
            face20 = block2['X'][:,0],block2['Y'][:,0],block2['Z'][:,0]
            face21 = block2['X'][:,-1],block2['Y'][:,-1],block2['Z'][:,-1]
        elif dim == 2:
            face1 = block1['X'][:,:,pos],block1['Y'][:,:,pos],block1['Z'][:,:,pos]
            face20 = block2['X'][:,:,0],block2['Y'][:,:,0],block2['Z'][:,:,0]
            face21 = block2['X'][:,:,-1],block2['Y'][:,:,-1],block2['Z'][:,:,-1]
        else:
            raise Exception('dim Error')
        if is_equal_face(face1,face20):
            return 0
        elif is_equal_face(face1,face21):
            return -1
        else:
            return None
    dim1 = None
    for i in range(3):
        for j in [0,-1]:
            t = detectEqualFace(i,j)
            if t is not None:
                dim1,pos1,dim2,pos2 = i,j,i,t
    if dim1 is None:
        raise Exception('No detect equal face')
    assert pos1 == -1 and pos2 == 0
    blocks.pop(b2)
    h,w,d = shape2
    kk = [0,0,0]
    kk[dim2] = 1
    h1,w1,d1 = kk
    block1['X'] = np.concatenate((block1['X'],block2['X'][h1:h,w1:w,d1:d]),axis = dim2)
    block1['Y'] = np.concatenate((block1['Y'],block2['Y'][h1:h,w1:w,d1:d]),axis = dim2)
    block1['Z'] = np.concatenate((block1['Z'],block2['Z'][h1:h,w1:w,d1:d]),axis = dim2)
    if one2oneData:
        def deal_one2one(rr,b1,b2):
            bb1,rr1,bb2,rr2 = rr
            bb1 -= 1
            bb2 -= 1
            result = []
            length = shape1[dim1]-1
            delete_face = False
            for b,r in [(bb1,rr1),(bb2,rr2)]:
                if b > b2:
                    b = b - 1 
                elif b == b2:
                    b = b1 
                    ise,jse,kse,v1,v2 = r
                    if dim2 == 0:
                        
                        if ise[0] == 1 and ise[1] == 1:
                            delete_face = True
                        ise = ise[0]+length, ise[1]+length
                    elif dim2 == 1:
                        
                        if jse[0] == 1 and jse[1] == 1:
                            delete_face = True
                        jse = jse[0]+length, jse[1]+length
                    elif dim2 == 2:
                        if kse[0] == 1 and kse[1] == 1:
                            delete_face = True
                        kse = kse[0]+length, kse[1]+length
                        
                    r = ise,jse,kse,v1,v2
                result.extend((b+1,r))
            if delete_face:
                return None 
            else:
                return result
        result_one2one = [deal_one2one(i,b1,b2) for i in one2oneData]
        result_one2one = [i for i in result_one2one if i]
        Varify_one2one(blocks,result_one2one)
    else:
        result_one2one = None
    if patchedData:
        t = b1,b2,shape1,shape2,dim1
        mfun = mergedPatchedData
        patchedData2 = [(mfun(i[0],t),[mfun(ff,t) for ff in i[1]]) for i in patchedData]
    else:
        patchedData2 = None
    if boundaryData:
        boundaryData2 = dict()
        for key,value in boundaryData.items():
            blockid = int(key[:-2])-1
            if blockid == b2:
                assert value['bctype'] != 2005
                continue 
            elif blockid == b1:
                assert value['bctype'] != 2005
                continue
            elif blockid > b2:
                blockid -= 1
            key = '{}{}'.format(blockid+1,key[-2:])
            boundaryData2[key] = value
    else:
        boundaryData2 = None
    return blocks,result_one2one, patchedData2, boundaryData2
def mergedPatchedData(element_patchedData,mergedData):
    (blockId,faceId),xie_eta = element_patchedData[:2]
    if mergedData:
        if faceId[0].lower() == 'i':
            dimf = 0
        elif faceId[0].lower() == 'j':
            dimf = 1
        else:
            dimf = 2
    else:
        return element_patchedData

    targetId,mergeId,(h1,w1,d1),(h2,w2,d2),dim1 = mergedData

    if blockId > mergeId:
        blockId = blockId - 1
    elif blockId == mergeId:
        blockId = targetId
        h,w,d = h1+h2-1, w1+w2 -1, d1+d2-1
        if dim1 is 0:
            h,w,d = h1+h2-1,w1,d1 
            w1,d1 = 1,1
        elif dim1 is 1:
            h,w,d = h1,w1+w2-1,d1 
            h1,d1 = 1,1
        else:
            h,w,d = h1,w1,d1+d2-1
            h1,w1 = 1,1
        if dim1 != dimf:
            if dimf is 0:
                #xie = j  eta = k 
                xie_eta = w1,w,d1,d
            elif dimf is 1:
                #xie = k  eta = i
                xie_eta = d1,d,h1,h
            elif dimf is 2:
                #xie = j eta = i
                xie_eta = w1,w,h1,h
    elif blockId == targetId:
        blockId = targetId
        if dim1 != dimf:
            if dimf is 0 :
                #xie = j  eta = k 
                xie_eta = 1,w1,1,d1
            elif dimf is 1:
                #xie = k  eta = i
                xie_eta = 1,d1,1,h1
            elif dimf is 2:
                #xie = j eta = i
                xie_eta = 1,w1,1,h1
    
    if len(element_patchedData) == 2:
        return (blockId, faceId), xie_eta
    elif len(element_patchedData) ==3:
        return  (blockId, faceId), xie_eta,element_patchedData[2]
    else:
        raise Exception('Error 2342354')

def tetrahedronArea(p1,p2,p3,p4):
    px,py,pz = p4
    v1x,v1y,v1z = p1[0] - px, p1[1] - py, p1[2] - pz
    v2x,v2y,v2z = p2[0] - px, p2[1] - py, p2[2] - pz
    v3x,v3y,v3z = p3[0] - px, p3[1] - py, p3[2] - pz
    tx = v1y*v2z - v2y*v1z 
    ty = v1z*v2x - v2z*v1x 
    tz = v1x*v2y - v1y*v2x 
    volume = tx*v3x + ty*v3y + tz*v3z 
    return volume/6
def hexahedronArea(p1,p2,p3,p4,p5,p6,p7,p8):
    v  = tetrahedronArea(p2,p4,p5,p1)
    v += tetrahedronArea(p7,p2,p5,p6)
    v += tetrahedronArea(p4,p7,p5,p8)
    v += tetrahedronArea(p4,p2,p7,p3)
    v += tetrahedronArea(p7,p5,p2,p4)
    return v
def CheckBlockVolume(block):
    x,y,z = block['X'], block['Y'], block['Z']
    t = [(0,0,0),(1,0,0),(1,1,0),(0,1,0),
        (0,0,1),(1,0,1),(1,1,1),(0,1,1)]
    p = [(x[i],y[i],z[i]) for i in t]
    volume = hexahedronArea(*p)
    return volume
def AdjustNegativeGrid(blocks):
    return [{k:v[::-1,:,:] for k,v in b.items()} if CheckBlockVolume(b)<0 else b for b in blocks]
def patchedFace2Faces(blocks,face1,faces,faces_type,rotate_angle='positive',steady=False):
    if faces_type.lower() == 'periodic':
        _,f2 = PatchedInterfaceRotatedPeriodic(blocks,[face1],faces)
        rotate_angle = f2[1]*f2[2]
        to_face_result = face1,(0,0,0,0)
        from_face_result = [(faces[0],(0,0,0,0),rotate_angle)]
        return to_face_result,from_face_result
    face1_xyz = GetFace(blocks[face1[0]],face1[1])
    point0 = face1_xyz[0][-1,-1],face1_xyz[1][-1,-1],face1_xyz[2][-1,-1]
    faces_xyz = [GetFace(blocks[bid],key) for bid,key in faces]
    # k = GetFacesAngle(point0,faces_xyz,'Top')
    # print(k)
    face1_theta_top = GetFaceAngle(point0,face1_xyz,'Top')
    face1_theta_bottom = GetFaceAngle(point0,face1_xyz,'bottom')

    faces_theta_top = [GetFaceAngle(point0,i,'Top') for i in faces_xyz]
    faces_theta_bottom = [GetFaceAngle(point0,i,'bottom') for i in faces_xyz]

    theta_min = [i[-2] for i in faces_theta_top]
    theta_max = [i[-1] for i in faces_theta_top]
    theta_delta = [i[0] for i in faces_theta_top]
    Theta = sum(theta_delta)#整个faces的角度
    print(Theta,faces)
    print(max(theta_max) - min(theta_min))
    assert abs(max(theta_max) - min(theta_min) - Theta) < 0.00001

    faces_theta_top_extend = [(face_number,j,delta,amin+j*Theta,amax+j*Theta) for face_number,(delta,amin,amax) in enumerate(faces_theta_top) for j in range(-3,4)]
    faces_theta_bottom_extend = [(face_number,j,delta,amin+j*Theta,amax+j*Theta) for face_number,(delta,amin,amax) in enumerate(faces_theta_bottom) for j in range(-3,4)]
    if rotate_angle.lower() == 'positive':
        rotate_angle = Theta 
    elif rotate_angle.lower() == 'negative':
        rotate_angle = -Theta
    else:
        rotate_angle = float(rotate_angle)
        
    if steady:
        rotate_angle = 0

    if faces_type.lower() == 'rotor':
        rotate_angle *=-1
    pd,pmi,pma = face1_theta_top
    face1_theta_top_extend = [pd+abs(rotate_angle),min(pmi,pmi+rotate_angle),max(pma,pma+rotate_angle)]
    face1_theta_bottom_extend = [
        face1_theta_bottom[0]+abs(rotate_angle),
        min(face1_theta_bottom[1],face1_theta_bottom[1]+rotate_angle),
        max(face1_theta_bottom[2],face1_theta_bottom[2]+rotate_angle)
    ]
    def isPatched(element_faces,element_face1):
        _,_,_,mi,ma = element_faces 
        _,fmi,fma = element_face1 
        if fmi <= mi <= fma or fmi <= ma <= fma:
            return True
        else:
            return False
    f1 = [i for i in faces_theta_top_extend if isPatched(i,face1_theta_top_extend)]
    f2 = [i for i in faces_theta_bottom_extend if isPatched(i,face1_theta_bottom_extend)]
    from_faces = list({i[:2] for i in f1+f2})
    from_faces.sort(key = lambda x:faces_theta_top[x[0]][1]+x[1]*Theta)
    to_face_result = face1,(0,0,0,0)
    from_faces_result = [(faces[i],(0,0,0,0),j*Theta) for i,j in from_faces]
    return to_face_result,from_faces_result
def translatePatcheDataToString(blocks,pdata,ITOSS=0,ITMAX=0):
    if ITMAX is None:
        ITMAX = 0
    def getITMAX_ITOSS(faces,itmax=0):
        if itmax is not 0:
            return itmax,0
        blockId, faceId = faces[0]
        i,j,k = blocks[blockId]['X'].shape
        key = faceId[0].lower()
        if key == 'i':
            t = max(j,k)
        elif key == 'j':
            t = max(i,k)
        else:
            t = max(i,j)
        if t>itmax:
            itmax = t 
        return t,0
    title = '  DYNAMIC PATCH INPUT DATA\n    NINTER\n{:>10d}\n   INT  IFIT    LIMIT    ITMAX    MCXIE    MCETA      C-0    IORPH    ITOSS\n'
    title_block = [getITMAX_ITOSS(to_face,ITMAX) for to_face,_ in pdata]
    NINTER = len(pdata)
    result = title.format(NINTER)
    fstr1 = '{:>6d}'*2+'{:>9d}'*7+'\n'
    for i,(itmax,itoss) in enumerate(title_block):
        result += fstr1.format(i+1,1,1,itmax,0,0,0,0,itoss)

    fstr2 = '   INT    TO     XIE1     XIE2     ETA1     ETA2      NFB\n'
    fstr2 += '{:>6d}'*2+'{:>9d}'*5+'\n'
    translate_table = str.maketrans('ijkIJK0eE','123123122')

    fromStr1 = '        FROM     XIE1     XIE2     ETA1     ETA2    FACTJ    FACTK\n'
    fromStr1 += '{:>12d}'+'{:>9d}'*4+'{:>9.4f}{:9.4f}\n'
    dxStr = '          DX       DY       DZ   DTHETX   DTHETY   DTHETZ\n'
    dxStr += '{:>12.4f}'+'{:>9.4f}'*5+'\n'
    fromStr = fromStr1+dxStr

    strings_result = [result]
    for i,(((toid,tokey),(xie1,xie2,eta1,eta2)),from_faces) in enumerate(pdata):
        TO = '{}{}'.format(toid+1,tokey.translate(translate_table))
        if xie1 != 0:
            print((xie1,xie2,eta1,eta2),toid,tokey)
        strings_result.append(fstr2.format(i+1,int(TO),xie1,xie2,eta1,eta2,len(from_faces)))
        for (fromId,faceId),(xie1,xie2,eta1,eta2),delta_theta in from_faces:
            FROM = '{}{}'.format(fromId+1,faceId.translate(translate_table))
            delta_theta = delta_theta/3.1415926*180
            t = fromStr.format(int(FROM),xie1,xie2,eta1,eta2,0,0,0,0,0,0,0,delta_theta)
            strings_result.append(t)
    return ''.join(strings_result)
if __name__=='__main__':
    #检查one2one中设置周期面时候的序号问题 放假之前没有解决
    from yxspkg import plot3d
    # t = plot3d.read('')
    t = plot3d.read('/home/yxs/F/learning/CFL3D/CFL3D/Stage-B_SA_pb99240-6.7.2/Stage_B_250w.g')
    # t = plot3d.read('/home/yxs/F/pythonAPP/cfl3d_pyui/test.g')
    # t = [t[13],t[25]]
    # t = plot3d.read('/home/yxs/F/pythonAPP/cfl3d_pyui/RS_work/rotate323_mesh.g')
    # '''
    # (([(2, 'KE'), (4, 'KE'), (5, 'KE'), (7, 'KE'), (9, 'KE'), (10, 'KE'), (12, 'KE'), (14, 'KE'), (15, 'KE')], 27.69230767779229, 0, 3), 
    # ([(16, 'K0'), (19, 'K0'), (20, 'IE'), (22, 'K0'), (25, 'K0'), (26, 'IE')], 27.6923076964478, -2, 1))
    # '''
    # x = patchedFace2Faces(t,(9,'KE'),[(15, 'K0'), (18, 'K0'), (19, 'IE'), (21, 'K0'), (24, 'K0'), (25, 'IE')],'rotor')
    # print(x)
    # x = patchedFace2Faces(t,(15,'IE'),[(14,'IE')],'periodic')
    # k = translatePatcheDataToString(t,[x])
    # print(k)
    # periodic_faces=[((0,'I0'),(0,'IE'),False),((1,'I0'),(1,'IE'),False),((2,'IE'),(5,'IE'),False)]
    # periodic_faces = None
    k = one2one(t)
    print(len(k))
    # q = transfer_one2one_str(k)
    # print(q)
    # print(oneData == oneData2,'ererere')
    #patchedSurface
    # p1 = PatchedInterfaceRotated(t,[(14,'Ie'),], [(15,'Ie'),],0,0,True)
    # p2 = PatchedInterfaceRotated(t,[(1,'K0'),(3,'K0'),(4,'Ke')], [(5,'Ke'),(8,'Ke'),(9,'Ie')],0,1)
    # p3 = PatchedInterfaceRotated(t,[(11,'Ke'),(14,'Ke'),(15,'K0')], [(6,'K0'),(8,'K0'),(10,'Ke')],0,1)
    # patchedData = [[BlockIdPlus(t) for t in p] for p in [p1,p2,p3]]
    # for i in patchedData:
    #     print(i)
    # m = patched(patchedData[2:3])
    # for i in m:
    #     print(i[0])
    # print(m)
    # blocks,_,pd = MergeBlocks(t,9,10,one2oneData=None,patchedData=m)
    # print('##################################################################')
    # print(pd)
    # for (f1,f2),fs in pd:
    #     print(f1,f2,len(fs))
    # # zz = translatePatcheDataToString(blocks,pd)
    # # print(zz)
    # hexahedronArea(
    #     (0,0,0),(1,0,0),(1,1,0),(0,1,0),
    #     (0,0,1),(1,0,1),(1,1,1),(0,1,1)
    # )
    # t = AdjustNegativeGrid(t)
    # AdjustNegativeGrid(t)


import sys 
import os

def tianhe():
    for i in sys.argv[1:]:
        if i.find('blsc605:')!=-1:
            break 
    else:
        return False
    command = ' '.join(sys.argv[1:]).replace('blsc605:','blsc605@gz:/PARA/blsc605/')
    command = '/home/yxs/F/learning/tianhe/papp_cloud-2.6.11/papp_cloud_linux64 rsync -i /home/yxs/F/learning/tianhe/papp_cloud-2.6.11/blsc605.id '+command 
    print(command)
    os.system(command)
    return True

if __name__=='__main__':
    is_tianhe = tianhe()


import hashlib,sys
from pathlib import Path
def main(filenames = None):
    if filenames is None:
        filenames = sys.argv[1:]
    for filename in filenames:
        fp = open(filename,'rb')
        md5 = hashlib.md5()
        while True:
            t = fp.read(1024*1024*4)
            if t == b'':
                break
            md5.update(t)
        print(Path(filename).name,md5.hexdigest())
if __name__=='__main__':
    main()


import sys
import matplotlib
matplotlib.use("Qt5Agg")
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from PyQt5.QtWidgets import QSizePolicy, QTextEdit
from matplotlib.figure import Figure
from functions import Residual

class MplCanvas(FigureCanvas):
    """这是一个窗口部件，即QWidget（当然也是FigureCanvasAgg）"""
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        fig = Figure(figsize=(width, height), dpi=dpi)
        self.fig = fig
        # self.axes = fig.add_subplot(111)

        self.compute_initial_figure()
        FigureCanvas.__init__(self, fig)
        self.setParent(parent)

        FigureCanvas.setSizePolicy(self,
                                   QSizePolicy.Expanding,
                                   QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)

    def compute_initial_figure(self):
        pass
    
class DrawResidual:
    def __init__(self,fig,filename):
        self.axes = fig.add_subplot(111)
        
        self.set_residual_file(filename)
    def set_residual_file(filename):
        self.res = Residual(filename)
        self.filename = filename
        
class SongziShell(QTextEdit):
    pass


import sys 
import json
import os
import shutil
from os import path
def copy_data(inpdata,jdata,tag):
    inpfile = jdata['inpfile']
    dirname = ''.join(inpfile.split('.')[:-1])+'_{:04d}'.format(tag)
    if not path.exists(dirname):
        os.mkdir(dirname)
    inp = path.join(dirname,inpfile)
    fp = open(inp,'w')
    fp.write(''.join(inpdata))
    meshfile = inpdata[1].strip()
    restartfile = inpdata[13].strip()
    shutil.copy(meshfile,path.join(dirname,meshfile))
    shutil.copy(restartfile,path.join(dirname,restartfile))
    shutil.copy('./cfl3d_mpi',dirname)
    return dirname,inpfile
def run_cfl3d(dirname,inp):
    pwd = os.getcwd()
    os.chdir(dirname)
    os.system('mpirun -np 4 ./cfl3d_mpi < {}'.format(inp))
    os.chdir(pwd)
def set_bc(bctype,bcdata,inpdata,jdata,tag):
    n = -1
    while n < len(inpdata)-10:
        n += 1
        line = inpdata[n].split()
        if len(line) <8: continue
        if not line[2].isdigit():continue
        if int(line[2]) == int(bctype):
            inpdata[n+2] = bcdata.strip()+'\n'
            n += 2
    # print(inpdata)
    return copy_data(inpdata,jdata,tag)

def run(jfile):
    jdata = json.loads(open(jfile).read())
    inpdata = open(jdata['inpfile']).readlines()
    tag = 0
    for k in jdata:
        if k.startswith('bc'):
            bctype = k[2:]
            for bcdata in jdata[k]:
                tag += 1
                dirname,inp = set_bc(bctype,str(bcdata),inpdata,jdata,tag)
                run_cfl3d(dirname,inp)
if __name__=='__main__':
    run('./test.json')